#!/usr/bin/env python

from gnuradio import gr, gr_unittest, blocks
import grdab

class qa_sum_elements_vff(gr_unittest.TestCase):
	"""
	@brief QA for the vector element adder block.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_sum_elements_vff(self):
		a = [1,2,3,4,5]
		b = [-1000,1000,0,-1000,1000]
		src_data        = a+b
		expected_result = [sum(a),sum(b)]
		src = blocks.vector_source_f(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_float, 5)
		sum_elements_vff = grdab.sum_elements_vff(5)
		dst = blocks.vector_sink_f()
		self.tb.connect(src, s2v, sum_elements_vff, dst)
		self.tb.run()
		result_data = dst.data()
		# print expected_result
		# print result_data
		self.assertFloatTuplesAlmostEqual(expected_result, result_data, 8)
	
	def test_002_sum_elements_vff(self):
		a = range(-1000,3096)
		b = range(10000,14096)
		src_data        = a+b
		expected_result = [sum(a),sum(b)]
		src = blocks.vector_source_f(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_float, 4096)
		sum_elements_vff = grdab.sum_elements_vff(4096)
		dst = blocks.vector_sink_f()
		self.tb.connect(src, s2v, sum_elements_vff, dst)
		self.tb.run()
		result_data = dst.data()
		# print expected_result
		# print result_data
		self.assertFloatTuplesAlmostEqual(expected_result, result_data, 8)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab

class qa_insert_null_symbol(gr_unittest.TestCase):
	"""
	@brief QA for Null symbol insertion.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_insert_null_symbol(self):
		src_data0       = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15]
		src_data1       = [0,1,0,1,0]
		expected_result = [1,2,3,0,0,0,0,0,4,5,6,7,8,9,0,0,0,0,0,10,11,12,13,14,15]
		expected_result = [complex(x) for x in expected_result]
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		s2v = blocks.stream_to_vector(gr.sizeof_gr_complex, 3)
		insert_null_symbol = grdab.insert_null_symbol(5,3)
		dst = blocks.vector_sink_c()
		self.tb.connect(src0, s2v, insert_null_symbol, dst)
		self.tb.connect(src1, (insert_null_symbol,1))
		self.tb.run()
		result_data = dst.data()
		# print expected_result
		# print result_data
		self.assertComplexTuplesAlmostEqual(expected_result, result_data, 6)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest, blocks
import grdab

class qa_qpsk_demapper_vcb(gr_unittest.TestCase):
	"""
	@brief QA for the QPSK demapper.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_qpsk_demapper_vcb(self):
		src_data        = [1+2j,3+1j,-1+1j,-1+1j,-0.0001+1000j,1+1j,1+1j,1+1j]
		expected_result = (10,128)
		src = blocks.vector_source_c(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_gr_complex, 4)
		qpsk_demapper_vcb = grdab.qpsk_demapper_vcb(4)
		v2s = blocks.vector_to_stream(gr.sizeof_char, 1)
		dst = blocks.vector_sink_b()
		self.tb.connect(src, s2v, qpsk_demapper_vcb, v2s, dst)
		self.tb.run()
		result_data = dst.data()
		# print expected_result
		# print result_data
		self.assertEqual(expected_result, result_data)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python
# _*_ coding: utf8 _*_

# Andreas Müller, 2008
# andrmuel@ee.ethz.ch
#
# this code may be freely used under GNU GPL conditions

"""
demodulate DAB signal and ouput to constellation sink
"""

from gnuradio import gr, blocks
import osmosdr
from gnuradio import eng_notation
from gnuradio.eng_option import eng_option
from gnuradio.wxgui import stdgui2, fftsink2, scopesink2
import grdab
from optparse import OptionParser
import wx
import sys, threading, time

class usrp_dab_gui_rx(stdgui2.std_top_block):
	def __init__(self, frame, panel, vbox, argv):
		stdgui2.std_top_block.__init__(self, frame, panel, vbox, argv)

		self.frame = frame
		self.panel = panel

		parser = OptionParser(option_class=eng_option, usage="%prog: [options] <filename>")
  		parser.add_option("-m", "--dab-mode", type="int", default=1,
        	     	help="DAB mode [default=%default]")
		parser.add_option("-F", "--filter-input", action="store_true", default=False,
                          help="Enable FFT filter at input")
		parser.add_option('-c', '--correct-ffe', action="store_true", default=False,
		     help="do fine frequency correction")
		parser.add_option('-u', '--correct-ffe-usrp', action="store_true", default=False,
		     help="do fine frequency correction by retuning the USRP instead of in software")
		parser.add_option('-e', '--equalize-magnitude', action="store_true", default=False,
		     help="do magnitude equalization")
  		parser.add_option("-s", "--resample-fixed", type="eng_float", default=1,
			help="resample by a fixed factor (fractional interpolation)")
		parser.add_option("-S", "--autocorrect-sample-rate", action="store_true", default=False,
                          help="Estimate sample rate offset and resample (dynamic fractional interpolation)")
		parser.add_option("-R", "--rx-subdev-spec", type="subdev", default=(0, 0),
		     help="select USRP Rx side A or B [default=A]")
		parser.add_option("-f", "--freq", type="eng_float", default=227.36e6,
		     help="set frequency to FREQ [default=%default]")
		parser.add_option("-p", "--ppm", type="int", default=0,
		     help="set frequency correction in ppm [default=%default]")
		parser.add_option("-r", "--sample-rate", type="int", default=2000000,
		     help="set sample rate to SAMPLE_RATE [default=%default]")
		parser.add_option("-d", "--decim", type="intx", default=32,
		     help="set decimation rate to DECIM [default=%default]")
		parser.add_option("-g", "--rx-gain", type="eng_float", default=None,
		     help="set receive gain in dB (default is midpoint)")
		parser.add_option('-v', '--verbose', action="store_true", default=False,
		     help="verbose output")
		(options, args) = parser.parse_args()

		
		self.verbose = options.verbose
                self.sample_rate = options.sample_rate

		if len(args) == 0:
			if self.verbose:
				print "--> receiving from RTL-SDR"
			self.src = osmosdr.source()
			self.src.set_sample_rate(self.sample_rate)
			self.src.set_freq_corr(options.ppm)
			# tune frequency
			self.frequency = options.freq
			self.set_freq(options.freq)

			# set gain      
                        if options.rx_gain is None:
                                # if no gain was specified, use AGC
                                self.src.set_gain_mode(True, 0)
                        else:
                                self.src.set_gain(options.rx_gain, 0)

		else:
			if self.verbose:
				print "--> receiving from file: " + args[0]
			self.filename = args[0]
			self.src = blocks.file_source(gr.sizeof_gr_complex, self.filename, False)
		
		
		self.dab_params = grdab.parameters.dab_parameters(mode=options.dab_mode, sample_rate=self.sample_rate, verbose=options.verbose)
		self.rx_params = grdab.parameters.receiver_parameters(mode=options.dab_mode, softbits=True, input_fft_filter=options.filter_input, autocorrect_sample_rate=options.autocorrect_sample_rate, sample_rate_correction_factor=options.resample_fixed, verbose=options.verbose, correct_ffe=options.correct_ffe, equalize_magnitude=options.equalize_magnitude)
	
		
		self.demod = grdab.ofdm_demod(self.dab_params, self.rx_params, verbose=self.verbose) 

		self.v2s = blocks.vector_to_stream(gr.sizeof_gr_complex, self.dab_params.num_carriers)
		self.scope = scopesink2.scope_sink_c(self.panel, title="DAB constellation sink", sample_rate=self.dab_params.sample_rate, xy_mode=True)

		self.sink = blocks.null_sink(gr.sizeof_float*self.dab_params.num_carriers*2)

		self.connect(self.src, self.demod, self.sink)
        
		# build GUI
		self.connect(self.demod.deinterleave, self.v2s, self.scope)
		vbox.Add(self.scope.win, 10, wx.EXPAND)

                # FFT Sink
		self.wxgui_fftsink2_0 = fftsink2.fft_sink_c(
			self.panel,
			baseband_freq=0,
			y_per_div=10,
			y_divs=10,
			ref_level=0,
			ref_scale=2.0,
			sample_rate=self.sample_rate,
			fft_size=1024,
			fft_rate=15,
			average=False,
			avg_alpha=None,
			title="FFT Plot",
			peak_hold=False,
		)
		vbox.Add(self.wxgui_fftsink2_0.win)
		self.connect((self.src, 0), (self.wxgui_fftsink2_0, 0))
		
		# retune USRP to correct FFE?
		self.correct_ffe_usrp = options.correct_ffe_usrp
		if self.correct_ffe_usrp:
			print "--> correcting FFE on USRP"
			self.run_correct_ffe_thread = True
			self.ffe_updater = threading.Timer(0.1, self.correct_ffe)
			self.ffe_updater.setDaemon(True)
			self.ffe_updater.start()

	def correct_ffe(self):
		while self.run_correct_ffe_thread:
			diff = self.demod.sync.ffs_sample_and_average_arg.ffe_estimate()
			if abs(diff) > self.rx_params.usrp_ffc_min_deviation:
				self.frequency -= diff*self.rx_params.usrp_ffc_adapt_factor
				print "--> updating fine frequency correction: " + str(self.frequency)
				self.set_freq(self.frequency)
			time.sleep(1./self.rx_params.usrp_ffc_retune_frequency)


	def set_freq(self, freq):
		if self.src.set_center_freq(freq, 0):
			if self.verbose:
				print "--> retuned to " + str(freq) + " Hz"
			return True
		else:
			print "-> error - cannot tune to " + str(freq) + " Hz"
			return False


if __name__ == '__main__':
	app = stdgui2.stdapp(usrp_dab_gui_rx, "usrp_dab_gui_rx", nstatus=1)
	app.MainLoop()






#
# Copyright 2008,2009 Free Software Foundation, Inc.
#
# This application is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This application is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# The presence of this file turns this directory into a Python package

'''
This is the GNU Radio DAB module. Place your Python package
description here (python/__init__.py).
'''

from __future__ import unicode_literals

# import swig generated symbols into the dab namespace
try:
	# this might fail if the module is python-only
	from .grdab_swig import *
except ImportError:
	pass

# import any pure python here
from .ofdm_sync_dab import ofdm_sync_dab
from .ofdm_sync_dab2 import ofdm_sync_dab2
from .detect_null import detect_null
from .parameters import dab_parameters
from .parameters import receiver_parameters
from .ofdm import ofdm_mod
from .ofdm import ofdm_demod
from .fic import fic_decode
from .msc_decode import msc_decode
from .dabplus_audio_decoder_ff import dabplus_audio_decoder_ff
from .dab_audio_decoder_ff import dab_audio_decoder_ff
from .osmo_or_zmq_source import osmo_or_zmq_source
from . import constants
#



#!/usr/bin/env python

from gnuradio import gr, gr_unittest, blocks
import grdab
import cmath

class qa_unpuncture_vff(gr_unittest.TestCase):
	"""
	@brief QA for the unpuncturing block.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_unpuncture_vff(self):
		src_data = (0,1,2,3,4,5,6,7,8,9)
		punc_seq = (1,0,0,0,1,0,1,1,1)
		exp_res  = (0,77,77,77,1,77,2,3,4,5,77,77,77,6,77,7,8,9)
		src = blocks.vector_source_f(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_float, 5)
		unpuncture_vff = grdab.unpuncture_vff(punc_seq, 77)
		v2s = blocks.vector_to_stream(gr.sizeof_float, 9)
		dst = blocks.vector_sink_f()
		self.tb.connect(src, s2v, unpuncture_vff, v2s, dst)
		self.tb.run()
		result_data = dst.data()
		self.assertFloatTuplesAlmostEqual(exp_res, result_data)
	
	def test_002_unpuncture_vff(self):
		src_data = (0,1,2,3,4,5,6,7,8,9)
		punc_seq = (1,0,0,0,1,0,1,1,1)
		exp_res  = (0,0,0,0,1,0,2,3,4,5,0,0,0,6,0,7,8,9)
		src = blocks.vector_source_f(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_float, 5)
		unpuncture_vff = grdab.unpuncture_vff(punc_seq)
		v2s = blocks.vector_to_stream(gr.sizeof_float, 9)
		dst = blocks.vector_sink_f()
		self.tb.connect(src, s2v, unpuncture_vff, v2s, dst)
		self.tb.run()
		result_data = dst.data()
		self.assertFloatTuplesAlmostEqual(exp_res, result_data)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab
import cmath
from math import pi

class qa_ofdm_ffs_sample(gr_unittest.TestCase):
	"""
	@brief Module test for the OFDM ffs sampler with phase calculation.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_ofdm_ffe_all_in_one(self):
		symbol_length = 5
		num_symbols = 2
		fft_length = 3
		alpha = 0.1
		src_data0 = [77*cmath.exp(2j*pi*x/20) for x in range(0,20)]
		src_data0[4] = 100 # should not matter, as this area of the symbol is not evaluated
		src_data0[12:19] = [100] * 7 # should not matter, as only the first two symbols are evaluated
		src_data1 =       (0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)
		d = -2*pi/20 # phase diff between two consective samples
		expected_result = (0,0,0,0,0,0,0,0,0,0,0,0,d,d,d,d,d,d,d,d)
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		ffe = grdab.ofdm_ffe_all_in_one(symbol_length, fft_length, num_symbols, alpha, 10)
		dst0 = blocks.vector_sink_f()
		self.tb.connect(src0, (ffe,0))
		self.tb.connect(src1, (ffe,1))
		self.tb.connect(ffe, dst0)
		self.tb.run()
		result_data0 = dst0.data()
		# print expected_result
		# print result_data0
		self.assertFloatTuplesAlmostEqual(expected_result, result_data0, 3)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab

class qa_moving_sum(gr_unittest.TestCase):
	"""
	@brief Module test for the moving sum class.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_moving_sum_ff(self):
		src_data = (0,1,3,3,-3.5,-7.7,2,2,3)
		expected_result = (0,1,4,7,2.5,-8.2,-9.2,-3.7,7)
		src = blocks.vector_source_f(src_data)
		moving_sum = grdab.moving_sum_ff(3)
		dst = blocks.vector_sink_f()
		self.tb.connect(src, moving_sum)
		self.tb.connect(moving_sum, dst)
		self.tb.run()
		result_data = dst.data()
		self.assertFloatTuplesAlmostEqual(expected_result, result_data, 6)

	def test_002_moving_sum_ff(self):
		src_data = [float(i**3)*(7**-2) for i in range(-20,20)]
		expected_result = [src_data[0]]+[src_data[i]+src_data[i-1] for i in range(1,40)]
		src = blocks.vector_source_f(src_data)
		moving_sum = grdab.moving_sum_ff(2)
		dst = blocks.vector_sink_f()
		self.tb.connect(src, moving_sum)
		self.tb.connect(moving_sum, dst)
		self.tb.run()
		result_data = dst.data()
		self.assertFloatTuplesAlmostEqual(expected_result, result_data, 4)

	def test_001_moving_sum_cc(self):
		src_data = (0j,1+0j,1j,-1+0j,0j,0j,0j,1+0j,1j,2+0j)
		expected_result = (0j,1+0j,1+1j,1j,1j,1j,-1+1j,0j,1+1j,3+1j)
		src = blocks.vector_source_c(src_data)
		moving_sum = grdab.moving_sum_cc(5)
		dst = blocks.vector_sink_c()
		self.tb.connect(src, moving_sum)
		self.tb.connect(moving_sum, dst)
		self.tb.run()
		result_data = dst.data()
		self.assertComplexTuplesAlmostEqual(expected_result, result_data, 6)

	def test_002_moving_sum_cc(self):
		src_data = [float(i**3)*(7**-2)+0.5j*i for i in range(-20,20)]
		expected_result = [src_data[0]]+[src_data[i]+src_data[i-1] for i in range(1,40)]
		src = blocks.vector_source_c(src_data)
		moving_sum = grdab.moving_sum_cc(2)
		dst = blocks.vector_sink_c()
		self.tb.connect(src, moving_sum)
		self.tb.connect(moving_sum, dst)
		self.tb.run()
		result_data = dst.data()
		self.assertComplexTuplesAlmostEqual(expected_result, result_data, 4)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python
# _*_ coding: utf8 _*_

# Andreas Müller, 2008
# andrmuel@ee.ethz.ch
#
# this code may be freely used under GNU GPL conditions

"""
Complete DAB TX and RX with a software channel to simulate noise, frequency offset, etc.
"""

from gnuradio import gr, blks2
import grdab
import math, random

DAB_SAMPLE_RATE=2048000

class dab_ofdm_testbench(gr.top_block):
	def __init__(self, autocorrect_sample_rate=False, input_filter=True, ber_sink=False):
		gr.top_block.__init__(self)
		self.dp = grdab.dab_parameters(1)
		self.rp = grdab.receiver_parameters(1, softbits=False, input_fft_filter=input_filter, autocorrect_sample_rate=autocorrect_sample_rate, correct_ffe=True, equalize_magnitude=False)
		self.ber_sink = ber_sink
		os.environ['GR_SCHEDULER'] = "STS" # need single threaded scheduler for use with concatenate_signals

	def setup_flowgraph(self, mode, ber_skipbytes=0):
		# parameters
		self.dp.set_mode(mode)
		self.rp.set_mode(mode)
		self.vlen = self.dp.num_carriers/4
		self.ber_skipbytes = ber_skipbytes

		# trigger signals
		frame_trigger = [1]+[0]*(self.dp.symbols_per_frame-2)
		self.frame_start = frame_trigger*(len(self.random_bytes)/(self.vlen*(self.dp.symbols_per_frame-1)))+frame_trigger[0:(len(self.random_bytes)/self.vlen)%(self.dp.symbols_per_frame-1)]

		# sources/sinks
		self.source    = gr.vector_source_b(self.random_bytes, False)
		self.trig      = gr.vector_source_b(self.frame_start, False)
		if self.ber_sink:
			self.sink = grdab.blocks.measure_ber_b()
		else:
			self.sink = gr.vector_sink_b()

		# self.noise_start      = gr.noise_source_c(gr.GR_GAUSSIAN, math.sqrt(2), random.randint(0,10000))
		# self.noise_start_head = gr.head(gr.sizeof_gr_complex, NOISE_SAMPLES_AT_START)
		# self.noise_end        = gr.noise_source_c(gr.GR_GAUSSIAN, math.sqrt(2), random.randint(0,10000))
		# self.noise_end_head   = gr.head(gr.sizeof_gr_complex, NOISE_SAMPLES_AT_END)
		
		# blocks
		self.s2v       = gr.stream_to_vector(gr.sizeof_char, self.vlen)
		self.v2s       = gr.vector_to_stream(gr.sizeof_char, self.vlen)
		if self.ber_sink:
			self.ber_skipbytes0 = gr.skiphead(gr.sizeof_char, self.ber_skipbytes)
			self.ber_skipbytes1 = gr.skiphead(gr.sizeof_char, self.ber_skipbytes+self.dp.bytes_per_frame)
		
		# more blocks (they have state, so better reinitialise them)
		self.mod       = grdab.ofdm_mod(self.dp, debug = False)
		self.rescale   = gr.multiply_const_cc(1)
		self.amp       = gr.multiply_const_cc(1)
		self.channel   = blks2.channel_model(noise_voltage=0, noise_seed=random.randint(0,10000))
		# self.cat       = grdab.concatenate_signals(gr.sizeof_gr_complex)
		self.demod     = grdab.ofdm_demod(self.dp, self.rp, debug = False, verbose = True)

		# connect it all
		if self.ber_sink:
			self.connect(self.source, self.s2v, (self.mod,0), self.rescale, self.amp, self.channel, (self.demod,0), self.v2s, self.ber_skipbytes0, self.sink)
			self.connect(self.source, self.ber_skipbytes1, (self.sink,1))
		else:
			self.connect(self.source, self.s2v, (self.mod,0), self.rescale, self.amp, self.channel, (self.demod,0), self.v2s, self.sink)
		self.connect(self.trig, (self.mod,1))

		# SNR calculation and prober
		self.probe_signal = gr.probe_avg_mag_sqrd_c(0,0.00001)
		self.probe_total  = gr.probe_avg_mag_sqrd_c(0,0.00001)
		self.connect(self.amp, self.probe_signal)
		self.connect(self.channel, self.probe_total)
        	
	def gen_random_bytes(self, num_bytes):
		self.random_bytes = [random.randint(0,255) for i in xrange(0,num_bytes)]
		
	def set_noise_energy(self, noise_energy):
		self.channel.set_noise_voltage(math.sqrt(noise_energy)/math.sqrt(2))

	def set_signal_energy(self, signal_energy):
		self.amp.set_k(math.sqrt(signal_energy))
	
	def set_carrier_frequency_offset(self, freq_offset):
		self.channel.set_frequency_offset(float(freq_offset)/self.dp.sample_rate)

	def set_sampling_frequency_offset(self, ratio):
		self.channel.set_timing_offset(ratio)	

	def set_multipath_taps(self, taps):
		self.channel.set_taps(taps)

	def set_power_correction(self, estimated_energy):
		self.estimated_signal_energy = estimated_energy
		sig_ratio = float(self.dp.num_carriers) / float(self.dp.fft_length) # not all subcarriers are occupied -> must be considered to calculate scale factor
		self.rescale.set_k((1/math.sqrt(self.estimated_signal_energy))*math.sqrt(sig_ratio))

	def rewind_sources(self):
		self.source.rewind()
		self.trig.rewind()
		# self.noise_start_head.rewind()
		# self.noise_end_head.rewind()
		
	def clear_sinks(self):
		self.sink.clear()
	
	def clear_state(self):
		if self.ber_sink:
			self.ber_skipbytes0.rewind()
			self.ber_skipbytes1.rewind()
		# self.cat.reset()
		self.demod.clear_state()

		# TODO some state is still left in the demod block - for now just make a new one
		self.disconnect(self.channel, self.demod)
		self.disconnect((self.demod,0), self.v2s)
		self.demod = grdab.ofdm_demod(self.dp, self.rp, debug = False, verbose = True)
		self.connect(self.channel, self.demod)
		self.connect((self.demod,0), self.v2s)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2017 Moritz Luca Schmid, Communications Engineering Lab (CEL) / Karlsruhe Institute of Technology (KIT).
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab_swig as grdab

class qa_unpuncture_ff (gr_unittest.TestCase):

    def setUp (self):
        self.tb = gr.top_block ()

    def tearDown (self):
        self.tb = None

    def test_001_unpuncture_ff(self):
        src_data = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
        punc_seq = (1, 0, 0, 0, 1, 0, 1, 1, 1)
        exp_res = (0, 77, 77, 77, 1, 77, 2, 3, 4, 5, 77, 77, 77, 6, 77, 7, 8, 9)
        src = blocks.vector_source_f(src_data)
        unpuncture_ff = grdab.unpuncture_ff(punc_seq, 77)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, unpuncture_ff, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(exp_res, result_data)

    def test_002_unpuncture_ff(self):
        src_data = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
        punc_seq = (1, 0, 0, 0, 1, 0, 1, 1, 1)
        exp_res = (0, 0, 0, 0, 1, 0, 2, 3, 4, 5, 0, 0, 0, 6, 0, 7, 8, 9)
        src = blocks.vector_source_f(src_data)
        unpuncture_ff = grdab.unpuncture_ff(punc_seq, 0)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, unpuncture_ff, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(exp_res, result_data)


if __name__ == '__main__':
    gr_unittest.run(qa_unpuncture_ff, "qa_unpuncture_ff.xml")



#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab
import cmath

class qa_ofdm_coarse_frequency_correct(gr_unittest.TestCase):
	"""
	@brief QA for the coarse frequency correction class.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_ofdm_coarse_frequency_correct(self):
		fft_length   = 10
		num_carriers = 2
		cp_length    = 3
		src_data0        = [0,1,2,3,4,5,6,7,8,9,1,2,0,5,7,6,0,4,0,6,1,1,1,0.8,0.1,1.3,1,0.7,1,1,0,1,2,3,4,5,6,7,8,9]
		expected_result0 = [7,9,5,6,0.8,1.3,3,5]
		offset           = [3,3,-1,-1,-1,-1,-1,-1]
		frame_index      = [0,0, 0, 0, 0, 0, 1, 1]
		expected_result0 = [complex(expected_result0[i])*cmath.exp(-2j*cmath.pi*offset[i]*cp_length/float(fft_length)*frame_index[i]) for i in range(0,8)]
		src_data1 = [1,1,1,0]
		expected_result1 = (1,1,1,0)
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		s2v0 = blocks.stream_to_vector(gr.sizeof_gr_complex, fft_length)
		ofdm_coarse_frequency_correct = grdab.ofdm_coarse_frequency_correct(fft_length,num_carriers,cp_length)
		v2s0 = blocks.vector_to_stream(gr.sizeof_gr_complex, num_carriers)
		dst0 = blocks.vector_sink_c()
		dst1 = blocks.vector_sink_b()
		self.tb.connect(src0, s2v0, (ofdm_coarse_frequency_correct,0))
		self.tb.connect(src1, (ofdm_coarse_frequency_correct,1))
		self.tb.connect((ofdm_coarse_frequency_correct,0), v2s0, dst0)
		self.tb.connect((ofdm_coarse_frequency_correct,1), dst1)
		self.tb.run()
		result_data0 = dst0.data()
		result_data1 = dst1.data()
		# print expected_result0
		# print result_data0
		self.assertComplexTuplesAlmostEqual(expected_result0, result_data0, 4)
		self.assertEqual(result_data1, expected_result1)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2018 Ruben Undheim
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import gr

class osmo_or_zmq_source(gr.hier_block2):
    """
    """

    def __init__(self, frequency=220.352e6, rf_gain=25, if_gain=0, bb_gain=0, use_zeromq=False, server="tcp://127.0.0.1:10444", server_control="tcp://127.0.0.1:10445"):

        gr.hier_block2.__init__(self,
                                "osmo_or_zmq_source",
                                # Input signature
                                gr.io_signature(0, 0, 1),
                                # Output signature
                                gr.io_signature(1, 1, gr.sizeof_gr_complex))

        samp_rate = 2000000
        self.use_zeromq = use_zeromq

        if not use_zeromq:
            import osmosdr
            self.osmosdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + '' )
            self.osmosdr_source_0.set_sample_rate(samp_rate)
            self.osmosdr_source_0.set_center_freq(frequency, 0)
            self.osmosdr_source_0.set_freq_corr(0, 0)
            self.osmosdr_source_0.set_dc_offset_mode(0, 0)
            self.osmosdr_source_0.set_iq_balance_mode(0, 0)
            self.osmosdr_source_0.set_gain_mode(False, 0)
            self.osmosdr_source_0.set_gain(rf_gain, 0)
            self.osmosdr_source_0.set_if_gain(if_gain, 0)
            self.osmosdr_source_0.set_bb_gain(bb_gain, 0)
            self.osmosdr_source_0.set_antenna('', 0)
            self.osmosdr_source_0.set_bandwidth(2000000, 0)
            self.src = self.osmosdr_source_0
        else:
            from gnuradio import zeromq
            self.zeromq_source = zeromq.sub_source(gr.sizeof_gr_complex, 1, server, 100, False, -1)
            self.rpc_mgr_server = zeromq.rpc_manager()
            self.rpc_mgr_server.set_request_socket(server_control)
            self.rpc_mgr_server.request("set_sample_rate",[samp_rate])
            self.rpc_mgr_server.request("set_rf_gain",[rf_gain])
            self.rpc_mgr_server.request("set_if_gain",[if_gain])
            self.rpc_mgr_server.request("set_bb_gain",[bb_gain])
            self.rpc_mgr_server.request("set_ppm",[0]) # Not using hardware correction since it behaves differently on different hardware
            self.rpc_mgr_server.request("set_frequency",[frequency])
            self.src = self.zeromq_source


        self.connect(self.src, (self, 0))


    def set_frequency(self, val):
        if self.use_zeromq:
            self.rpc_mgr_server.request("set_frequency",[val]) 
        else:
            self.osmosdr_source_0.set_center_freq(val, 0)

    def set_rf_gain(self, val):
        if self.use_zeromq:
            self.rpc_mgr_server.request("set_rf_gain",[val]) 
        else:
            self.osmosdr_source_0.set_gain(val, 0)

    def set_if_gain(self, val):
        if self.use_zeromq:
            self.rpc_mgr_server.request("set_if_gain",[val]) 
        else:
            self.osmosdr_source_0.set_if_gain(val, 0)

    def set_bb_gain(self, val):
        if self.use_zeromq:
            self.rpc_mgr_server.request("set_bb_gain",[val]) 
        else:
            self.osmosdr_source_0.set_bb_gain(val, 0)



#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab

class qa_frequency_interleaver_vcc(gr_unittest.TestCase):
	"""
	@brief QA for frequency interleaving.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_frequency_interleaver_vcc(self):
		src_data        = [0,1,2,3,4,5,6,7,8,9+1j]
		expected_result = [2,1,4,0,3,7,6,9+1j,5,8]
		expected_result = [complex(x) for x in expected_result]
		src = blocks.vector_source_c(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_gr_complex, 5)
		frequency_interleaver_vcc = grdab.frequency_interleaver_vcc([3,1,0,4,2])
		v2s = blocks.vector_to_stream(gr.sizeof_gr_complex, 5)
		dst = blocks.vector_sink_c()
		self.tb.connect(src, s2v, frequency_interleaver_vcc, v2s, dst)
		self.tb.run()
		result_data = dst.data()
		# print expected_result
		# print result_data
		self.assertComplexTuplesAlmostEqual(expected_result, result_data, 6)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest, blocks
import grdab

class qa_repartition_vectors(gr_unittest.TestCase):
	"""
	@brief QA for the vector repartitioning block

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_repartition_vectors(self):
		ilen = 3
		mult = 2
		div  = 3
		olen = 2
		src_data        = (0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6)
		trig            = (0,    1,    0,    0,       0,       0,    0,    0,    1,     1,    0)
		expected_data   = (3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,1,2,3,4,5,6)
		expected_trig   = (1,  0,  0,  0,    0,    0,    0,  0,  0,  1,  0,  0)
		src = blocks.vector_source_b(src_data)
		trigsrc = blocks.vector_source_b(trig)
		s2v = blocks.stream_to_vector(gr.sizeof_char, ilen)
		repartition_vectors = grdab.repartition_vectors(gr.sizeof_char,ilen,olen,mult,div)
		v2s = blocks.vector_to_stream(gr.sizeof_char, olen)
		dst = blocks.vector_sink_b()
		trigdst = blocks.vector_sink_b()
		self.tb.connect(src, s2v, repartition_vectors, v2s, dst)
		self.tb.connect(trigsrc, (repartition_vectors,1), trigdst)
		self.tb.run()
		result_data = dst.data()
		result_trig = trigdst.data()
		self.assertEqual(expected_data, result_data)
		self.assertEqual(expected_trig, result_trig)
	
	def test_002_repartition_vectors(self):
		ilen = 3
		mult = 2
		div  = 3
		olen = 2
		src_data        = (0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6)
		trig            = (0,    1,    0,    0,       0,       0,    0,    0,    1,     1,    0)
		expected_data   = (3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,1,2,3,4,5,6)
		expected_trig   = (1,  0,  0,  0,    0,    0,    0,  0,  0,  1,  0,  0)
		src = blocks.vector_source_f(src_data)
		trigsrc = blocks.vector_source_b(trig)
		s2v = blocks.stream_to_vector(gr.sizeof_float, ilen)
		repartition_vectors = grdab.repartition_vectors(gr.sizeof_float,ilen,olen,mult,div)
		v2s = blocks.vector_to_stream(gr.sizeof_float, olen)
		dst = blocks.vector_sink_f()
		trigdst = blocks.vector_sink_b()
		self.tb.connect(src, s2v, repartition_vectors, v2s, dst)
		self.tb.connect(trigsrc, (repartition_vectors,1), trigdst)
		self.tb.run()
		result_data = dst.data()
		result_trig = trigdst.data()
		self.assertFloatTuplesAlmostEqual(expected_data, result_data)
		self.assertEqual(expected_trig, result_trig)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python
#
# Copyright 2008 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

# Andreas Mueller, 2008
# andrmuel@ee.ethz.ch

from gnuradio import gr, trellis, blocks
from . import grdab_swig as grdab
from math import sqrt

"""
DAB FIC layer
"""

class fic_decode(gr.hier_block2):
    """
    @brief block to decode FIBs (fast information blocks) from the FIC (fast information channel) of a demodulated DAB signal

    - get FIBs from byte stream
    - do convolutional decoding
    - undo energy dispersal
    - get FIC information
    """

    def __init__(self, dab_params, verbose=False, debug=False):
        """
        Hierarchical block for FIC decoding

        @param dab_params DAB parameter object (grdab.parameters.dab_parameters)
        """
        gr.hier_block2.__init__(self, "fic",
                                gr.io_signature(1, 1, gr.sizeof_float * dab_params.num_carriers * 2),
                                gr.io_signature(1, 1, gr.sizeof_char * 32))

        self.dp = dab_params
        self.verbose = verbose
        self.debug = debug

        # FIB selection and block partitioning
        self.select_fic_syms = grdab.select_vectors(gr.sizeof_float, self.dp.num_carriers * 2, self.dp.num_fic_syms, 0)
        self.repartition_fic = grdab.repartition_vectors(gr.sizeof_float, self.dp.num_carriers * 2,
                                                       self.dp.fic_punctured_codeword_length, self.dp.num_fic_syms,
                                                       self.dp.num_cifs)

        # unpuncturing
        self.unpuncture = grdab.unpuncture_vff(self.dp.assembled_fic_puncturing_sequence, 0)

        # convolutional coding
        # self.fsm = trellis.fsm(self.dp.conv_code_in_bits, self.dp.conv_code_out_bits, self.dp.conv_code_generator_polynomials)
        self.fsm = trellis.fsm(1, 4, [0o133, 0o171, 0o145, 0o133])  # OK (dumped to text and verified partially)
        self.conv_v2s = blocks.vector_to_stream(gr.sizeof_float, self.dp.fic_conv_codeword_length)
        # self.conv_decode = trellis.viterbi_combined_fb(self.fsm, 20, 0, 0, 1, [1./sqrt(2),-1/sqrt(2)] , trellis.TRELLIS_EUCLIDEAN)
        table = [
            0, 0, 0, 0,
            0, 0, 0, 1,
            0, 0, 1, 0,
            0, 0, 1, 1,
            0, 1, 0, 0,
            0, 1, 0, 1,
            0, 1, 1, 0,
            0, 1, 1, 1,
            1, 0, 0, 0,
            1, 0, 0, 1,
            1, 0, 1, 0,
            1, 0, 1, 1,
            1, 1, 0, 0,
            1, 1, 0, 1,
            1, 1, 1, 0,
            1, 1, 1, 1
        ]
        assert (len(table) / 4 == self.fsm.O())
        table = [(1 - 2 * x) / sqrt(2) for x in table]
        self.conv_decode = trellis.viterbi_combined_fb(self.fsm, 774, 0, 0, 4, table, trellis.TRELLIS_EUCLIDEAN)
        #self.conv_s2v = blocks.stream_to_vector(gr.sizeof_char, 774)
        self.conv_prune = grdab.prune(gr.sizeof_char, self.dp.fic_conv_codeword_length / 4, 0,
                                            self.dp.conv_code_add_bits_input)

        # energy dispersal
        self.prbs_src = blocks.vector_source_b(self.dp.prbs(self.dp.energy_dispersal_fic_vector_length), True)
        #self.energy_v2s = blocks.vector_to_stream(gr.sizeof_char, self.dp.energy_dispersal_fic_vector_length)
        self.add_mod_2 = blocks.xor_bb()
        self.energy_s2v = blocks.stream_to_vector(gr.sizeof_char, self.dp.energy_dispersal_fic_vector_length)
        self.cut_into_fibs = grdab.repartition_vectors(gr.sizeof_char, self.dp.energy_dispersal_fic_vector_length,
                                                     self.dp.fib_bits, 1, self.dp.energy_dispersal_fic_fibs_per_vector)

        # connect all
        self.nullsink = blocks.null_sink(gr.sizeof_char)
        self.pack = blocks.unpacked_to_packed_bb(1, gr.GR_MSB_FIRST)
        self.fibout = blocks.stream_to_vector(1, 32)
        # self.filesink = gr.file_sink(gr.sizeof_char, "debug/fic.dat")
        self.fibsink = grdab.fib_sink_vb()

        # self.connect((self,0), (self.select_fic_syms,0), (self.repartition_fic,0), self.unpuncture, self.conv_v2s, self.conv_decode, self.conv_s2v, self.conv_prune, self.energy_v2s, self.add_mod_2, self.energy_s2v, (self.cut_into_fibs,0), gr.vector_to_stream(1,256), gr.unpacked_to_packed_bb(1,gr.GR_MSB_FIRST), self.filesink)
        self.connect((self, 0),
                     (self.select_fic_syms, 0),
                     (self.repartition_fic, 0),
                     self.unpuncture,
                     self.conv_v2s,
                     self.conv_decode,
                     #self.conv_s2v,
                     self.conv_prune,
                     #self.energy_v2s,
                     self.add_mod_2,
                     self.energy_s2v,
                     (self.cut_into_fibs, 0),
                     blocks.vector_to_stream(1, 256),
                     self.pack,
                     self.fibout,
                     self.fibsink)
        self.connect(self.fibout, self)
        self.connect(self.prbs_src, (self.add_mod_2, 1))

        if self.debug:
            self.connect((self, 0), blocks.file_sink(gr.sizeof_float * self.dp.num_carriers * 2, "debug/transmission_frame.dat"))
            self.connect((self, 1), blocks.file_sink(gr.sizeof_char, "debug/transmission_frame_trigger.dat"))
            self.connect(self.select_fic_syms, blocks.file_sink(gr.sizeof_float * self.dp.num_carriers * 2, "debug/fic_select_syms.dat"))
            self.connect(self.repartition_fic, blocks.file_sink(gr.sizeof_float * self.dp.fic_punctured_codeword_length, "debug/fic_repartitioned.dat"))
            self.connect(self.unpuncture, blocks.file_sink(gr.sizeof_float * self.dp.fic_conv_codeword_length, "debug/fic_unpunctured.dat"))
            self.connect(self.conv_decode, blocks.file_sink(gr.sizeof_char, "debug/fic_decoded.dat"))
            self.connect(self.conv_prune, blocks.file_sink(gr.sizeof_char, "debug/fic_decoded_pruned.dat"))
            #self.connect(self.conv_decode, blocks.file_sink(gr.sizeof_char * self.dp.energy_dispersal_fic_vector_length, "debug/fic_energy_dispersal_undone.dat"))
            self.connect(self.pack, blocks.file_sink(gr.sizeof_char, "debug/fic_energy_undone.dat"))

    def get_ensemble_info(self):
        return self.fibsink.get_ensemble_info()

    def get_service_info(self):
        return self.fibsink.get_service_info()

    def get_service_labels(self):
        return self.fibsink.get_service_labels()

    def get_subch_info(self):
        return self.fibsink.get_subch_info()

    def get_programme_type(self):
        return self.fibsink.get_programme_type()

    def get_crc_passed(self):
        return self.fibsink.get_crc_passed()

    def set_print_channel_info(self, val):
        self.fibsink.set_print_channel_info(val)



#!/usr/bin/env python

from gnuradio import gr, gr_unittest, blocks
import grdab

class qa_sum_phasor_trig_vcc(gr_unittest.TestCase):
	"""
	@brief QA for the phase summation class.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_sum_phasor_trig_vcc(self):
		src_data0        = (1,1j,1,1j,1j,1j,-1, 1j, 1,1j,1,1j)
		src_data1        = (0,   1,   0,     0,     1,   0)
		expected_result0 = (0,0, 1,1j,1j,-1,-1j,-1j,1,1j,1,-1)
		expected_result0 = [x+0j for x in expected_result0]
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		s2v0 = blocks.stream_to_vector(gr.sizeof_gr_complex,2)
		sum_phasor_trig_vcc = grdab.sum_phasor_trig_vcc(2)
		v2s0 = blocks.vector_to_stream(gr.sizeof_gr_complex,2)
		dst0 = blocks.vector_sink_c()
		dst1 = blocks.vector_sink_b()
		self.tb.connect(src0, s2v0, (sum_phasor_trig_vcc,0))
		self.tb.connect(src1, (sum_phasor_trig_vcc,1))
		self.tb.connect((sum_phasor_trig_vcc,0), v2s0, dst0)
		self.tb.connect((sum_phasor_trig_vcc,1), dst1)
		self.tb.run()
		result_data0 = dst0.data()
		result_data1 = dst1.data()
		# print expected_result0
		# print result_data0
		self.assertComplexTuplesAlmostEqual(expected_result0, result_data0, 6)
		self.assertEqual(result_data1, src_data1)

	def test_002_sum_phasor_trig_vcc(self):
		src_data0        = (1,1j,-1,1,1)*50 # try it multiple times to decect problems when leaving the function in between
		src_data1        = (1,0,0,1,0)*50
		expected_result0 = (1,1j,-1j,1,1)*50
		expected_result0 = [x+0j for x in expected_result0]
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		s2v0 = blocks.stream_to_vector(gr.sizeof_gr_complex,1)
		sum_phasor_trig_vcc = grdab.sum_phasor_trig_vcc(1)
		v2s0 = blocks.vector_to_stream(gr.sizeof_gr_complex,1)
		dst0 = blocks.vector_sink_c()
		dst1 = blocks.vector_sink_b()
		self.tb.connect(src0, s2v0, (sum_phasor_trig_vcc,0))
		self.tb.connect(src1, (sum_phasor_trig_vcc,1))
		self.tb.connect((sum_phasor_trig_vcc,0), v2s0, dst0)
		self.tb.connect((sum_phasor_trig_vcc,1), dst1)
		self.tb.run()
		result_data0 = dst0.data()
		result_data1 = dst1.data()
		self.assertComplexTuplesAlmostEqual(expected_result0, result_data0, 6)
		self.assertEqual(result_data1, src_data1)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest, blocks
import grdab

class qa_prune_vectors(gr_unittest.TestCase):
	"""
	@brief QA for the vector pruning block.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_prune_vectors(self):
		src_data        = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15] 
		expected_result = [3,4,8,9,13,14]
		src = blocks.vector_source_f(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_float, 5)
		prune_vectors = grdab.prune_vectors(gr.sizeof_float,5,2,1)
		v2s = blocks.vector_to_stream(gr.sizeof_float, 2)
		dst = blocks.vector_sink_f()
		self.tb.connect(src, s2v, prune_vectors, v2s, dst)
		self.tb.run()
		result_data = dst.data()
		# print expected_result
		# print result_data
		self.assertFloatTuplesAlmostEqual(expected_result, result_data, 6)

	def test_002_prune_vectors(self):
		src_data        = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15] 
		expected_result = [3,4,8,9,13,14]
		src = blocks.vector_source_b(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_char, 5)
		prune_vectors = grdab.prune_vectors(gr.sizeof_char,5,2,1)
		v2s = blocks.vector_to_stream(gr.sizeof_char, 2)
		dst = blocks.vector_sink_b()
		self.tb.connect(src, s2v, prune_vectors, v2s, dst)
		self.tb.run()
		result_data = dst.data()
		# print expected_result
		# print result_data
		self.assertFloatTuplesAlmostEqual(expected_result, result_data, 6)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab

class qa_measure_processing_rate(gr_unittest.TestCase):
	"""
	@brief QA for measure processing rate sink.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_measure_processing_rate(self):
		src = blocks.null_source(gr.sizeof_gr_complex)
		throttle = blocks.throttle(gr.sizeof_gr_complex, 1000000)
		head = blocks.head(gr.sizeof_gr_complex, 200000)
		sink = grdab.measure_processing_rate(gr.sizeof_gr_complex,100000)
		
		self.tb.connect(src, throttle, head, sink)
		self.tb.run()

		rate = sink.processing_rate()
		assert(rate > 900000 and rate < 1100000)
	
	def test_002_measure_processing_rate(self):
		src = blocks.null_source(gr.sizeof_char)
		throttle = blocks.throttle(gr.sizeof_char, 10000000)
		head = blocks.head(gr.sizeof_char, 1000000)
		sink = grdab.measure_processing_rate(gr.sizeof_char,1000000)
		
		self.tb.connect(src, throttle, head, sink)
		self.tb.run()

		rate = sink.processing_rate()
		assert(rate > 8000000 and rate < 12000000)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python2

def receive_dabplus(frequency=220.352e6, rf_gain=25, if_gain=0, bb_gain=0, ppm=80, audio_sample_rate=48000, dab_bit_rate=64, dab_address=304, dab_subch_size=64, dab_protect_level=1, use_zeromq=False, dabplus=True, server="tcp://127.0.0.1:10444", server_control="tcp://127.0.0.1:10445"):
    from gnuradio import gr, blocks, audio
    if use_zeromq:
        from gnuradio import zeromq

    import time
    import osmosdr
    import grdab

    samp_rate = samp_rate = 2000000

    print("Setting frequency: %0.3f MHz" % (frequency/1e6))
    print("Setting RF gain to: %d" % rf_gain)
    print("Setting Frequency error (ppm) to: %d" % ppm)

    if not use_zeromq:
        osmosdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + '' )
        osmosdr_source_0.set_sample_rate(samp_rate)
        osmosdr_source_0.set_center_freq(frequency, 0)
        osmosdr_source_0.set_freq_corr(0, 0)
        osmosdr_source_0.set_dc_offset_mode(0, 0)
        osmosdr_source_0.set_iq_balance_mode(0, 0)
        osmosdr_source_0.set_gain_mode(False, 0)
        osmosdr_source_0.set_gain(rf_gain, 0)
        osmosdr_source_0.set_if_gain(if_gain, 0)
        osmosdr_source_0.set_bb_gain(bb_gain, 0)
        osmosdr_source_0.set_antenna('RX2', 0)
        osmosdr_source_0.set_bandwidth(2000000, 0)
    else:
        zeromq_source = zeromq.sub_source(gr.sizeof_gr_complex, 1, server, 100, False, -1)
        rpc_mgr_server = zeromq.rpc_manager()
        rpc_mgr_server.set_request_socket(server_control)
        rpc_mgr_server.request("set_sample_rate",[samp_rate])
        rpc_mgr_server.request("set_rf_gain",[rf_gain])
        rpc_mgr_server.request("set_if_gain",[if_gain])
        rpc_mgr_server.request("set_bb_gain",[bb_gain])
        rpc_mgr_server.request("set_ppm",[0]) # Not using hardware correction since it behaves differently on different hardware
        rpc_mgr_server.request("set_frequency",[frequency])
        time.sleep(0.7)

    sample_rate_correction_factor = 1 + float(ppm)*1e-6
    dab_ofdm_demod_0 = grdab.ofdm_demod(
              grdab.parameters.dab_parameters(
                mode=1,
                sample_rate=samp_rate,
                verbose=False
              ),
              grdab.parameters.receiver_parameters(
                mode=1,
                softbits=True,
                input_fft_filter=True,
                autocorrect_sample_rate=False,
                sample_rate_correction_factor=sample_rate_correction_factor,
                always_include_resample=True,
                verbose=False,
                correct_ffe=True,
                equalize_magnitude=True
              )
            )

    if dabplus:
        decoder = grdab.dabplus_audio_decoder_ff(grdab.parameters.dab_parameters(mode=1, sample_rate=samp_rate, verbose=False), dab_bit_rate, dab_address, dab_subch_size, dab_protect_level, True)
    else:
        decoder = grdab.dab_audio_decoder_ff(grdab.parameters.dab_parameters(mode=1, sample_rate=samp_rate, verbose=False), dab_bit_rate, dab_address, dab_subch_size, dab_protect_level, True)

    xrun_monitor = grdab.xrun_monitor_cc(100000)
    f2c = blocks.float_to_complex()
    c2f = blocks.complex_to_float()

    audio_sink_0 = audio.sink(audio_sample_rate, '', True)

    fg = gr.top_block()

    if not use_zeromq:
        src = osmosdr_source_0
    else:
        src = zeromq_source

    fg.connect(src, dab_ofdm_demod_0, decoder)
    fg.connect((decoder, 0), (f2c, 0))
    fg.connect((decoder, 1), (f2c, 1))
    fg.connect(f2c, xrun_monitor)
    fg.connect(xrun_monitor, c2f)
    fg.connect((c2f, 0), (audio_sink_0, 0))
    fg.connect((c2f, 1), (audio_sink_0, 1))



    fg.start()
    input("Running..")
    #new = grdab.dabplus_audio_decoder_ff(grdab.parameters.dab_parameters(mode=1, sample_rate=samp_rate, verbose=False), 64, 304, 64, 1, True)
    #newaudio = audio.sink(44100, '', True)
    fg.stop()
    #fg.wait()
    #xrun_monitor.stop_until_tag()
    #fg.disconnect(src, dab_ofdm_demod_0, decoder)
    #fg.disconnect((decoder, 0), (f2c, 0))
    #fg.disconnect((decoder, 1), (f2c, 1))
    #fg.disconnect((c2f, 0), (audio_sink_0, 0))
    #fg.disconnect((c2f, 1), (audio_sink_0, 1))
    #decoder = new
    #audio_sink_0 = newaudio
    #fg.connect(src, dab_ofdm_demod_0, decoder)
    #fg.connect((decoder, 0), (f2c, 0))
    #fg.connect((decoder, 1), (f2c, 1))
    #fg.connect((c2f, 0), (audio_sink_0, 0))
    #fg.connect((c2f, 1), (audio_sink_0, 1))
    #fg.start()
    #raw_input("Running..")
    #fg.stop()



#!/usr/bin/env python
# _*_ coding: utf8 _*_

# Andreas Müller, 2008
# andrmuel@ee.ethz.ch
#
# this code may be freely used under GNU GPL conditions

"""
demodulate DAB signal and use it for calibration purposes
"""

from gnuradio import gr, blocks
import osmosdr
from gnuradio.eng_option import eng_option
import grdab
from optparse import OptionParser
import sys, threading, time

class rtl_sdr_dab_cal(gr.top_block):
	def __init__(self):
		gr.top_block.__init__(self)

		parser = OptionParser(option_class=eng_option, usage="%prog: [options] <filename>")
		parser.add_option("-m", "--dab-mode", type="int", default=1,
				help="DAB mode [default=%default]")
		parser.add_option("-F", "--filter-input", action="store_true", default=False,
				help="Enable FFT filter at input")
		parser.add_option('-c', '--correct-ffe', action="store_true", default=False,
			help="do fine frequency correction")
		parser.add_option('-u', '--correct-ffe-usrp', action="store_true", default=False,
			help="do fine frequency correction by retuning the USRP instead of in software")
		parser.add_option('-e', '--equalize-magnitude', action="store_true", default=False,
			help="do magnitude equalization")
		parser.add_option("-s", "--resample-fixed", type="eng_float", default=1,
			help="resample by a fixed factor (fractional interpolation)")
		parser.add_option("-S", "--autocorrect-sample-rate", action="store_true", default=False,
				help="Estimate sample rate offset and resample (dynamic fractional interpolation)")
		parser.add_option("-R", "--rx-subdev-spec", type="subdev", default=(0, 0),
			help="select USRP Rx side A or B [default=A]")
		parser.add_option("-f", "--freq", type="eng_float", default=227.36e6,
			help="set frequency to FREQ [default=%default]")
		parser.add_option("-r", "--sample-rate", type="int", default=2000000,
			help="set sample rate to SAMPLE_RATE [default=%default]")
		parser.add_option("-d", "--decim", type="intx", default=32,
			help="set decimation rate to DECIM [default=%default]")
		parser.add_option("-g", "--rx-gain", type="eng_float", default=None,
			help="set receive gain in dB (default is midpoint)")
		parser.add_option('-v', '--verbose', action="store_true", default=False,
			help="verbose output")
		(options, args) = parser.parse_args()

		
		self.verbose = options.verbose
		self.sample_rate = options.sample_rate

		if len(args) == 0:
			if self.verbose:
				print "--> receiving from USRP"
			self.src = osmosdr.source()
			self.src.set_sample_rate(self.sample_rate)
			self.src.set_freq_corr(0)
			# tune frequency
			self.frequency = options.freq
			self.orig_frequency = options.freq
			self.set_freq(options.freq)

			self.src.set_freq_corr(0)

			# set gain 
			if options.rx_gain is None:
				# if no gain was specified, use AGC
				self.src.set_gain_mode(True, 0)
			else:
				self.src.set_gain(options.rx_gain, 0)

		else:
			if self.verbose:
				print "--> receiving from file: " + args[0]
			self.filename = args[0]
			self.src = blocks.file_source(gr.sizeof_gr_complex, self.filename, False)
		
		
		self.dab_params = grdab.parameters.dab_parameters(mode=options.dab_mode, sample_rate=self.sample_rate, verbose=options.verbose)
		self.rx_params = grdab.parameters.receiver_parameters(mode=options.dab_mode, softbits=True, input_fft_filter=options.filter_input, 			autocorrect_sample_rate=options.autocorrect_sample_rate, sample_rate_correction_factor=options.resample_fixed, verbose=options.verbose, correct_ffe=options.correct_ffe, equalize_magnitude=options.equalize_magnitude)

		self.demod = grdab.ofdm_demod(self.dab_params, self.rx_params, verbose=self.verbose) 

		self.v2s = blocks.vector_to_stream(gr.sizeof_gr_complex, self.dab_params.num_carriers)

		self.sink = blocks.null_sink(gr.sizeof_float*self.dab_params.num_carriers*2)

		self.connect(self.src, self.demod, self.sink)

		# retune USRP to correct FFE?
		self.correct_ffe_usrp = options.correct_ffe_usrp
		if self.correct_ffe_usrp:
			print "--> correcting FFE on USRP"
			self.run_correct_ffe_thread = True
			self.ffe_updater = threading.Timer(0.1, self.correct_ffe)
			self.ffe_updater.setDaemon(True)
			self.ffe_updater.start()

	def correct_ffe(self):
		while self.run_correct_ffe_thread:
			carrier_offs = self.demod.cfs.get_delta_f() * 1000
			diff = self.demod.sync.ffe.ffe_estimate()
			error_hz = diff-carrier_offs
			error_ppm = (((self.orig_frequency-self.frequency)-error_hz) / self.frequency) * 1000000
			print "abs err: " + str(error_ppm) + " ppm\tsoft err: " + str(error_hz) + " Hz\tcarr: " + str(carrier_offs/1000)
			if abs(error_hz) > 500:
				self.frequency -= error_hz
				print "--> updating fine frequency correction: " + str(self.frequency)
				self.set_freq(self.frequency)
			time.sleep(1./self.rx_params.usrp_ffc_retune_frequency)


	def set_freq(self, freq):
		if self.src.set_center_freq(freq, 0):
			if self.verbose:
				print "--> retuned to " + str(freq) + " Hz"
			return True
		else:
			print "-> error - cannot tune to " + str(freq) + " Hz"
			return False

if __name__=='__main__':
	try:
		rx = rtl_sdr_dab_cal()
		rx.run()
	except KeyboardInterrupt:
		pass




#!/usr/bin/env python
# _*_ coding: utf8 _*_

# Andreas Müller, 2008
# andrmuel@ee.ethz.ch
#
# this code may be freely used under GNU GPL conditions

"""
Simulate channel with frequency offset
"""

import math, random, pylab
import dab_tb, ber

NUM_BYTES = 1000000

MODES=[1,2,3,4]
FREQ_OFFSET_RANGE = range(-300000,300000,10000)
FREQ_OFFSET_RANGE = range(0,1000,100)

PLOT_FORMAT=['-','-x','--x','-.x',':x']

# initialise test flowgraph
tb = dab_tb.dab_ofdm_testbench(ber_sink=True)
tb.gen_random_bytes(NUM_BYTES)

# prepeare plot	
#pylab.title("BER in channel with frequency shift") # add caption in LaTeX -> looks better
pylab.xlabel("Frequency shift [Hz]")
pylab.ylabel("BER")

# open logfile
logfile = open("freq_shift_ber_log.txt",'w')
logfile.write("number of bytes: " + str(NUM_BYTES) + "\nFrequency shift range: " + str(FREQ_OFFSET_RANGE) + "\n\n")

for mode in MODES:
	print "Mode: "+str(mode)+"\n-------\n"
	tb.setup_flowgraph(mode)
	ber_values = []
	bytes_received = []
	# estimate signal energy for this mode (disturbed by FFT ...)
	tb.set_signal_energy(1)
	tb.set_noise_energy(0)
	tb.run()
	tb.set_power_correction(tb.probe_signal.level())
	print "estimated energy: " + str(tb.probe_signal.level()) + "\n"
	for offset in FREQ_OFFSET_RANGE:
		print "Mode: "+str(mode)+" Offset: "+str(offset)
		# reset and run the test
		tb.rewind_sources()
		tb.clear_sinks()
		tb.clear_state()
		tb.set_carrier_frequency_offset(offset)
		tb.run()
		print "signal power: " + str(tb.probe_signal.level())
		print "total power: " + str(tb.probe_total.level())
		# get the result
		bytes_received.append(tb.sink.bytecount())
		print "bytes sent: " + str(NUM_BYTES)
		print "bytes received: " + str(tb.sink.bytecount())
		ber_values.append(tb.sink.ber())
		print "BER: " + str(ber_values[-1])
		print

	# write log
	logfile.write("Mode: " + str(mode)+"\n" +
		      "=======\n\n" +
		      "Frequency shift: BER (number of bytes received)\n" +
		      "-----------------------------------\n")
	for i in range(0,len(FREQ_OFFSET_RANGE)):
		logfile.write(str(FREQ_OFFSET_RANGE[i])+": "+str(ber_values[i])+" (" + str(bytes_received[i]) + ")\n")
	logfile.write("\n\n")
	# plot it:
	if sum([abs(x) for x in ber_values])==0:
		print "all BER values are 0 - not plotting"
	else:
		pylab.semilogy(FREQ_OFFSET_RANGE, ber_values, PLOT_FORMAT[mode], label="Mode "+str(mode))
		# pylab.plot(FREQ_OFFSET_RANGE, ber_values, PLOT_FORMAT[mode], label="Mode "+str(mode))

logfile.close()
# pylab.axis([FREQ_OFFSET_RANGE[0],FREQ_OFFSET_RANGE[-1],0,0.52])
pylab.legend()
pylab.show()



#!/usr/bin/env python
#
# Copyright 2008 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

# test_ofdm_sync_dab.py - test the DAB OFDM synchronisation
#
# Andreas Mueller, 2008
# andrmuel@ee.ethz.ch

from gnuradio import gr, blocks
from gnuradio.eng_option import eng_option
from optparse import OptionParser
from ofdm_sync_dab import ofdm_sync_dab
import parameters
import random
import os

class dab_ofdm_sync_test(gr.top_block):
	"""
	@brief Test application for the synchronisation block.
	"""
	def __init__(self):
		gr.top_block.__init__(self)
		
		usage = "%prog: [options] samples_file"
		parser = OptionParser(option_class=eng_option, usage=usage)
		(options, args) = parser.parse_args ()
		if len(args)<1:
			# print "using gaussian noise as source"
			# self.sigsrc = gr.noise_source_c(gr.GR_GAUSSIAN,10e6)
			print "using repeating random vector as source"
			self.sigsrc = blocks.vector_source_c([10e6*(random.random() + 1j*random.random()) for i in range(0,100000)],True)
			self.src = blocks.throttle( gr.sizeof_gr_complex,2048000)
			self.connect(self.sigsrc, self.src)
		else:
			filename = args[0]
			print "using samples from file " + filename
			self.src = blocks.file_source(gr.sizeof_gr_complex, filename, False)

		dp = parameters.dab_parameters(1)
		rp = parameters.receiver_parameters(1)
		self.sync_dab = ofdm_sync_dab(dp, rp, False)
		self.nop0 = blocks.nop(gr.sizeof_gr_complex)
		self.nop1 = blocks.nop(gr.sizeof_char)
		self.connect(self.src, self.sync_dab, self.nop0)
		self.connect((self.sync_dab,1), self.nop1)

if __name__=='__main__':
	try:
		dab_ofdm_sync_test().run()
	except KeyboardInterrupt:
		pass



#!/usr/bin/env python
#
# Copyright 2008 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

# Andreas Mueller, 2008
# andrmuel@ee.ethz.ch

class dab_parameters:
    """
    @brief Represents the DAB parameters.

    DAB parameters for mode I to IV

    as specified in
    ETSI EN 300 401 V1.4.1 (2006-06)
    "Digital Audio Broadcasting (DAB) to mobile, portable and fixed receivers"
    """

    # parameter values for all modes

    # OFDM parameters (section 14)
    # Table 38, page 145 of the DAB specification
    __symbols_per_frame__ = [76, 76, 153, 76]  # number of OFDM symbols per DAB frame (incl. pilot, excl. NS)
    __num_carriers__ = [1536, 384, 192, 768]  # number of carriers -> carrier width = 1536kHz/carriers
    __frame_length__ = [196608, 49152, 49152, 98304]  # samples per frame; in ms: 96,24,24,48 (incl. NS)
    __ns_length__ = [2656, 664, 345, 1328]  # length of null symbol in samples
    __symbol_length__ = [2552, 638, 319, 1276]  # length of an OFDM symbol in samples
    __fft_length__ = [2048, 512, 256, 1024]  # fft length
    __cp_length__ = [504, 126, 63, 252]  # length of cyclic prefix
    default_sample_rate = 2048000
    T = 1. / default_sample_rate

    # prn calculation data
    # tables 39-43 on pages 148 and 149

    # format: [mode][index][k_min, k_max, k', i, n]
    __prn_kin__ = [[
        [-768, -737, -768, 0, 1],
        [-736, -705, -736, 1, 2],
        [-704, -673, -704, 2, 0],
        [-672, -641, -672, 3, 1],
        [-640, -609, -640, 0, 3],
        [-608, -577, -608, 1, 2],
        [-576, -545, -576, 2, 2],
        [-544, -513, -544, 3, 3],
        [-512, -481, -512, 0, 2],
        [-480, -449, -480, 1, 1],
        [-448, -417, -448, 2, 2],
        [-416, -385, -416, 3, 3],
        [-384, -353, -384, 0, 1],
        [-352, -321, -352, 1, 2],
        [-320, -289, -320, 2, 3],
        [-288, -257, -288, 3, 3],
        [-256, -225, -256, 0, 2],
        [-224, -193, -224, 1, 2],
        [-192, -161, -192, 2, 2],
        [-160, -129, -160, 3, 1],
        [-128, -97, -128, 0, 1],
        [-96, -65, -96, 1, 3],
        [-64, -33, -64, 2, 1],
        [-32, -1, -32, 3, 2],
        [1, 32, 1, 0, 3],
        [33, 64, 33, 3, 1],
        [65, 96, 65, 2, 1],
        [97, 128, 97, 1, 1],
        [129, 160, 129, 0, 2],
        [161, 192, 161, 3, 2],
        [193, 224, 193, 2, 1],
        [225, 256, 225, 1, 0],
        [257, 288, 257, 0, 2],
        [289, 320, 289, 3, 2],
        [321, 352, 321, 2, 3],
        [353, 384, 353, 1, 3],
        [385, 416, 385, 0, 0],
        [417, 448, 417, 3, 2],
        [449, 480, 449, 2, 1],
        [481, 512, 481, 1, 3],
        [513, 544, 513, 0, 3],
        [545, 576, 545, 3, 3],
        [577, 608, 577, 2, 3],
        [609, 640, 609, 1, 0],
        [641, 672, 641, 0, 3],
        [673, 704, 673, 3, 0],
        [705, 736, 705, 2, 1],
        [737, 768, 737, 1, 1]
    ], [
        [-192, -161, -192, 0, 2],
        [-160, -129, -160, 1, 3],
        [-128, -97, -128, 2, 2],
        [-96, -65, -96, 3, 2],
        [-64, -33, -64, 0, 1],
        [-32, -1, -32, 1, 2],
        [1, 32, 1, 2, 0],
        [33, 64, 33, 1, 2],
        [65, 96, 65, 0, 2],
        [97, 128, 97, 3, 1],
        [129, 160, 129, 2, 0],
        [161, 192, 161, 1, 3]
    ], [
        [-96, -65, -96, 0, 2],
        [-64, -33, -64, 1, 3],
        [-32, -1, -32, 2, 0],
        [1, 32, 1, 3, 2],
        [33, 64, 33, 2, 2],
        [65, 96, 65, 1, 2]
    ], [
        [-384, -353, -384, 0, 0],
        [-352, -321, -352, 1, 1],
        [-320, -289, -320, 2, 1],
        [-288, -257, -288, 3, 2],
        [-256, -225, -256, 0, 2],
        [-224, -193, -224, 1, 2],
        [-192, -161, -192, 2, 0],
        [-160, -129, -160, 3, 3],
        [-128, -97, -128, 0, 3],
        [-96, -65, -96, 1, 1],
        [-64, -33, -64, 2, 3],
        [-32, -1, -32, 3, 2],
        [1, 32, 1, 0, 0],
        [33, 64, 33, 3, 1],
        [65, 96, 65, 2, 0],
        [97, 128, 97, 1, 2],
        [129, 160, 129, 0, 0],
        [161, 192, 161, 3, 1],
        [193, 224, 193, 2, 2],
        [225, 256, 225, 1, 2],
        [257, 288, 257, 0, 2],
        [289, 320, 289, 3, 1],
        [321, 352, 321, 2, 3],
        [353, 384, 353, 1, 0]
    ]]

    # h_i,j
    # note: values for h_i,j are the same as for h_i,j+16 ...
    __prn_h__ = [
        [0, 2, 0, 0, 0, 0, 1, 1, 2, 0, 0, 0, 2, 2, 1, 1, 0, 2, 0, 0, 0, 0, 1, 1, 2, 0, 0, 0, 2, 2, 1, 1],
        [0, 3, 2, 3, 0, 1, 3, 0, 2, 1, 2, 3, 2, 3, 3, 0, 0, 3, 2, 3, 0, 1, 3, 0, 2, 1, 2, 3, 2, 3, 3, 0],
        [0, 0, 0, 2, 0, 2, 1, 3, 2, 2, 0, 2, 2, 0, 1, 3, 0, 0, 0, 2, 0, 2, 1, 3, 2, 2, 0, 2, 2, 0, 1, 3],
        [0, 1, 2, 1, 0, 3, 3, 2, 2, 3, 2, 1, 2, 1, 3, 2, 0, 1, 2, 1, 0, 3, 3, 2, 2, 3, 2, 1, 2, 1, 3, 2]
    ]

    __expected_frequency_interleaving__ = [
        # these few values are listed in the specs - they are used to verify the sequence
        [-513, -14, 329, 692, -733, 13, 680, 273, -36, 43],
        [-129, -14, -55, -76, 163, 141, -88, 7, -111, -85],
        [-65, -14, 52, -29, -58, 77, 40, 71, -38, 81],
        [-257, -14, 73, 180, 198, -243, 168, 218, 17, 299]
    ]

    # transport mechanism parameters
    __num_fic_syms__ = [3, 3, 8, 3]  # number of OFDM symbols per frame belonging to the FIC
    __num_msc_syms__ = [72, 72, 144, 72] # number of OFDM symbols per frame belonging to the MSC

    # puncturing
    puncturing_vectors = [  # table 29, page 131
        [],  # "Who are you? How did you get in my house?"
        [1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0],
        # PI=1: code rate: 8/9
        [1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0],
        # PI=2: code rate: 8/10
        [1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0],
        # PI=3: code rate: 8/11
        [1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0],
        # PI=4: code rate: 8/12
        [1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0],
        # PI=5: code rate: 8/13
        [1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0],
        # PI=6: code rate: 8/14
        [1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0],
        # PI=7: code rate: 8/15
        [1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0],
        # PI=8: code rate: 8/16
        [1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0],
        # PI=9: code rate: 8/17
        [1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0],
        # PI=10 code rate: 8/18
        [1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0],
        # PI=11 code rate: 8/19
        [1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0],
        # PI=12 code rate: 8/20
        [1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0],
        # PI=13 code rate: 8/21
        [1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0],
        # PI=14 code rate: 8/22
        [1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0],
        # PI=15 code rate: 8/23
        [1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0],
        # PI=16 code rate: 8/24
        [1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0],
        # PI=17 code rate: 8/25
        [1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0],
        # PI=18 code rate: 8/26
        [1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0],
        # PI=19 code rate: 8/27
        [1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0],
        # PI=20 code rate: 8/28
        [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0],
        # PI=21 code rate: 8/29
        [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
        # PI=22 code rate: 8/30
        [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
        # PI=23 code rate: 8/31
        [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
        # PI=24 code rate: 8/32
    ]
    puncturing_tail_vector = [1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0]  # V_T
    puncturing_vectors_ones = [0, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32]
    # conv coding rate at a protection level, table 7
    conv_code_rate = [1/4, 3/8, 1/2, 3/4]
    subch_size_multiple_n = [12, 8, 6, 4]

    # convolutional coding - 11.1, page 129/130
    conv_code_generator_polynomials = [
        0o133,
        0o171,
        0o145,
        0o133
    ]
    conv_code_initial_state = 0
    conv_code_final_state = 0
    conv_code_constraint_length = 7
    conv_code_in_bits = 1
    conv_code_add_bits_input = 6
    conv_code_out_bits = 4
    __fic_conv_codeword_length__ = [3096, 3096, 4120, 3096]  # 4*I + 24
    __fic_punctured_codeword_length__ = [2304, 2304, 3072, 2304]

    # energy dispersal

    __energy_dispersal_fic_fibs_per_vector__ = [3, 3, 4, 3]
    __energy_dispersal_fic_vector_length__ = [768, 768, 1024, 768]  # I
    __prbs_bits__ = [0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
                     0]  # first 16 PRBS bits are given in the standard - can be used for another assert
    # time interleaving
    scrambling_vector = [0, 8, 4, 12, 2, 10, 6, 14, 1, 9, 5, 13, 3, 11, 7, 15]

    # transport mechanism parameters
    fib_bits = 256
    cif_bits = 55296
    __num_fibs__ = [12, 3, 4, 6]  # FIC
    __num_cifs__ = [4, 1, 1, 2]  # MSC -> num cifs = num fib groups
    num_cus = 864 # number of CUs in one cif
    msc_cu_size = 64 # size of capacity unit in msc (smallest unit)

    def __init__(self, mode, sample_rate=2048000, verbose=True):
        """
        selects the correct parameters for the selected mode and calculates the prn sequence, etc.

        @param mode DAB mode (I-IV)
        @param sample_rate sampling frequency
        """
        if verbose:
            print("--> creating DAB parameter object") # should not be seen more than once

        assert (mode >= 1 and mode <= 4)
        self.mode = mode
        self.sample_rate = sample_rate
        self.verbose = verbose

        # sanity checks:
        for i in range(0, 4):
            # OFDM parameters
            assert (
            self.__symbols_per_frame__[i] * self.__symbol_length__[i] + self.__ns_length__[i] == self.__frame_length__[
                i])
            assert (self.__symbol_length__[i] == self.__fft_length__[i] + self.__cp_length__[i])
            # block partitioning
            assert (self.__num_carriers__[i] * 2 * self.__num_fic_syms__[i] / self.__num_cifs__[i] ==
                    self.__fic_punctured_codeword_length__[i])
            # energy dispersal parameters
            assert (self.__energy_dispersal_fic_fibs_per_vector__[i] * self.fib_bits ==
                    self.__energy_dispersal_fic_vector_length__[i])
            assert (3 * self.__energy_dispersal_fic_vector_length__[i] == self.__fic_punctured_codeword_length__[
                i])  # not sure - according to specification, code rate is only approximately 1/3, but seems to be exact

        # sanity checks for PRBS sequence (energy dispersal)
        assert (self.prbs(16) == self.__prbs_bits__)  # bits from DAB standard
        assert (self.prbs(511) == self.prbs(1022)[511:])  # sequence must repeat itself
        if verbose:
            print("--> DAB parameters self check ok")

        self.__update_parameters__()

    def set_mode(self, mode):
        if self.verbose:
            print("--> setting DAB mode to " + str(mode))
        self.mode = mode
        self.__update_parameters__()

    def set_sample_rate(self, sample_rate):
        if self.verbose:
            print("--> setting sample rate to " + str(sample_rate))
        self.sample_rate = sample_rate
        self.__update_parameters__()

    def __update_parameters__(self):
        if self.verbose:
            print("--> updating DAB parameters")
        mode = self.mode
        # OFDM parameters (14)
        self.symbols_per_frame = self.__symbols_per_frame__[mode - 1]
        self.num_carriers = self.__num_carriers__[mode - 1]
        self.frame_length = self.__frame_length__[mode - 1]
        self.ns_length = self.__ns_length__[mode - 1]
        self.symbol_length = self.__symbol_length__[mode - 1]
        self.fft_length = self.__fft_length__[mode - 1]
        self.cp_length = self.__cp_length__[mode - 1]

        # bytes per frame and bytes per symbol
        self.bytes_per_frame = (self.symbols_per_frame - 1) * self.num_carriers / 4
        self.bytes_per_symbol = self.num_carriers / 4

        # prn sequence
        self.prn = []
        for k in range(-self.num_carriers // 2, self.num_carriers // 2 + 1):
            if k == 0:
                # self.prn.append(0)
                pass
            else:
                [kk, i, n] = self.__get_prn_kk_i_n__(k)
                h = self.__prn_h__[i][k - kk]
                phi_k = (h + n) % 4  # actually phi_k/(pi/2)
                if phi_k == 0:  # e^(j*pi/2*phi_k) is not exact if calculated by python
                    self.prn.append(1)
                elif phi_k == 1:
                    self.prn.append(1j)
                elif phi_k == 2:
                    self.prn.append(-1)
                elif phi_k == 3:
                    self.prn.append(-1j)

        # frequency (de)interleaving
        a = self.fft_length / 4 - 1
        b = self.fft_length
        A = [0]
        for i in range(1, self.fft_length):
            A.append((13 * A[-1] + a) % b)
        D = [d for d in A if d >= self.fft_length / 8 and d <= 7 * self.fft_length / 8 and d != self.fft_length / 2]
        assert (len(D) == self.num_carriers)
        self.frequency_interleaving_sequence = [d - self.fft_length / 2 for d in D]
        assert (self.frequency_interleaving_sequence[0:len(self.__expected_frequency_interleaving__[mode - 1])] ==
                self.__expected_frequency_interleaving__[mode - 1])
        # sequence for arrays, with indices starting from 0 and central carrier already removed
        self.frequency_interleaving_sequence_array = [k + self.num_carriers / 2 - (k > 0) for k in
                                                      self.frequency_interleaving_sequence]
        assert (len(self.frequency_interleaving_sequence_array) == self.num_carriers)
        assert (min(self.frequency_interleaving_sequence_array) == 0)
        assert (max(self.frequency_interleaving_sequence_array) == self.num_carriers - 1)
        assert (len(set(self.frequency_interleaving_sequence_array)) == len(
            self.frequency_interleaving_sequence_array))  # uniqueness of elements

        # frequency deinterleaving sequence
        self.frequency_deinterleaving_sequence_array = [self.frequency_interleaving_sequence_array.index(i) for i in
                                                        range(0, self.num_carriers)]

        # adapt for non-standard sample rate - do this at end, frequency interleaving calculation still needs default fft length
        if self.sample_rate != self.default_sample_rate:
            if self.verbose:
                print("--> using non-standard sample rate: " + str(self.sample_rate))
            self.T = 1. / self.sample_rate
            self.ns_length = int(round(self.ns_length * float(self.sample_rate) / float(self.default_sample_rate)))
            self.cp_length = int(round(self.cp_length * float(self.sample_rate) / float(self.default_sample_rate)))
            self.fft_length = int(round(self.fft_length * float(self.sample_rate) / float(self.default_sample_rate)))
            self.symbol_length = self.cp_length + self.fft_length
            self.frame_length = self.symbols_per_frame * self.symbol_length + self.ns_length

        # block partitioning parameters (14.4)
        self.num_fic_syms = self.__num_fic_syms__[mode - 1]
        self.num_msc_syms = self.__num_msc_syms__[mode - 1]

        # convolutional coding (11)
        self.fic_conv_codeword_length = self.__fic_conv_codeword_length__[mode - 1]  # length after puncturing

        # unpuncturing sequence (assembled, such that it can be applied on a complete fib group)
        # see 11.2 page 132
        self.fic_punctured_codeword_length = self.__fic_punctured_codeword_length__[mode - 1]
        if mode in [1, 2, 4]:
            self.assembled_fic_puncturing_sequence = 21 * 4 * self.puncturing_vectors[16] + 3 * 4 * \
                                                                                            self.puncturing_vectors[
                                                                                                15] + self.puncturing_tail_vector
        else:
            self.assembled_fic_puncturing_sequence = 29 * 4 * self.puncturing_vectors[16] + 3 * 4 * \
                                                                                            self.puncturing_vectors[
                                                                                                15] + self.puncturing_tail_vector
        assert (len(self.assembled_fic_puncturing_sequence) == self.fic_conv_codeword_length)
        #assert (
        #len(filter(lambda x: x == 1, self.assembled_fic_puncturing_sequence)) == self.fic_punctured_codeword_length)

        # energy dispersal (10)
        self.energy_dispersal_fic_fibs_per_vector = self.__energy_dispersal_fic_fibs_per_vector__[mode - 1]
        self.energy_dispersal_fic_vector_length = self.__energy_dispersal_fic_vector_length__[mode - 1]

        # transport mechanism parameters (5)
        self.num_fibs = self.__num_fibs__[mode - 1]
        self.num_cifs = self.__num_cifs__[mode - 1]

    def __get_prn_kk_i_n__(self, k):
        assert (k != 0)
        assert (abs(k) <= self.num_carriers // 2)
        if k < 0:
            index = (k + self.num_carriers // 2) // 32
            kk = 32 * (int(k) // 32)
        else:
            index = (k + self.num_carriers // 2 - 1) // 32
            kk = 32 * (int(k - 1) // 32) + 1
        values = self.__prn_kin__[self.mode - 1][index]
        assert (k >= values[0] and k <= values[1])
        assert (kk == values[2])
        i = values[3]
        n = values[4]
        return [kk, i, n]

    def prbs(self, length):
        """
        PRBS generated with the polynomial p(x) = x^9 + x^5 + 1
        and initial state 111111111

        @param length number of bits in the sequence
        """
        bits = [1] * 9
        sequence = []
        for i in range(0, length):
            newbit = bits[8] ^ bits[4]
            bits = [newbit] + bits[0:-1]
            sequence.append(newbit)
        return sequence

class receiver_parameters:
    """
    @brief Parameters for the receiver, independent of the DAB standard
    """
    # filter at input
    filt_bw = (768 + 100) * 1e3
    filt_tb = 50e3

    # OFDM stuff
    __cp_gap__ = [252, 63, 31, 124]  # gap for ofdm_sampler to leave before the start of the next symbol
    __symbols_for_ffs_estimation__ = [8, 8, 16, 8]  # number of symbols to evaluate for fine frequency error estimation
    __symbols_for_magnitude_equalization__ = [6, 6, 12,
                                              6]  # how many symbols should be used to estimate magnitude equalizer?
    ffs_alpha = 0.5

    # phase variance estimation
    phase_var_estimate_alpha = 0.01
    phase_var_estimate_downsample = 100  # 50 -> uses about 1% of the CPU time

    # for USRP
    usrp_ffc_retune_frequency = 5  # how often should the USRP be retuned at most?
    usrp_ffc_min_deviation = 5  # how far off does the FFE have to be to retune the USRP?
    usrp_ffc_adapt_factor = 0.5  # how much to adapt the correction?

    def __init__(self, mode, sample_rate=2048000, softbits=False, input_fft_filter=True, autocorrect_sample_rate=False,
                 sample_rate_correction_factor=1, correct_ffe=True, equalize_magnitude=True, verbose=True, always_include_resample=False):
        """
        Create new instance.

        @param mode DAB mode (I-IV)
        @param sample_rate sampling frequency
        @param input_fft_filter whether to use an FFT filter at the input
        @param autocorrect_sample_rate whether to correct the sample rate dynamically
        @param sample_rate_correction_factor static correction factor for sample rate
        @parem correct_ffe if False, only estimate fine frequency error - don't correct it
        @param verbose be talkative
        """
        if verbose:
            print("--> creating RX parameter object")
        assert (mode >= 1 and mode <= 4)

        self.set_mode(mode)
        self.sample_rate = sample_rate
        self.softbits = softbits
        self.input_fft_filter = input_fft_filter
        self.autocorrect_sample_rate = autocorrect_sample_rate
        self.sample_rate_correction_factor = sample_rate_correction_factor
        self.correct_ffe = correct_ffe
        self.equalize_magnitude = equalize_magnitude
        self.verbose = verbose
        self.always_include_resample = always_include_resample

    def set_mode(self, mode):
        self.mode = mode
        self.cp_gap = self.__cp_gap__[mode - 1]
        self.symbols_for_ffs_estimation = self.__symbols_for_ffs_estimation__[mode - 1]
        self.symbols_for_magnitude_equalization = self.__symbols_for_magnitude_equalization__[mode - 1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2017 Moritz Luca Schmid, Communications Engineering Lab (CEL) / Karlsruhe Institute of Technology (KIT).
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab_swig as grdab

class qa_prune (gr_unittest.TestCase):

    def setUp (self):
        self.tb = gr.top_block ()

    def tearDown (self):
        self.tb = None

    def test_001_prune(self):
        src_data = [1, 2, 3, 4, 5, 6, 7, 6, 5, 4, 3, 2, 1, 2, 3]
        expected_result = [3, 4, 6, 5, 1, 2]
        src = blocks.vector_source_b(src_data)
        prune = grdab.prune(gr.sizeof_char, 5, 2, 1)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, prune, dst)
        self.tb.run()
        result_data = dst.data()
        # print result_data
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 6)


if __name__ == '__main__':
    gr_unittest.run(qa_prune, "qa_prune.xml")



#!/usr/bin/env python3

import channel_mapping

def get_number_of_channels():
  return len(channel_mapping.table)

def id_to_frequency(i):
  frequency_mhz = float(channel_mapping.table[int(i)]['frequency'])
  print(frequency_mhz)
  return frequency_mhz*1e6



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2017 Moritz Luca Schmid, Communications Engineering Lab (CEL) / Karlsruhe Institute of Technology (KIT).
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import gr, gr_unittest
from gnuradio import blocks
from gnuradio import audio
import os
import grdab_swig as grdab

class qa_mp4_decode_bs (gr_unittest.TestCase):

    def setUp (self):
        self.tb = gr.top_block ()

    def tearDown (self):
        self.tb = None

# manual check, if header info makes sense, and if audio is played, input = repaired (= Reed-Solomon decoded) DAB+ audio superframes
    def test_001_t (self):
        if os.path.exists("debug/reed_solomon_repaired.dat"):
            self.src = blocks.file_source_make(gr.sizeof_char, "debug/reed_solomon_repaired.dat")
            self.mp4 = grdab.mp4_decode_bs_make(14)
            self.s2f_left = blocks.short_to_float_make(1, 32767)
            self.s2f_right = blocks.short_to_float_make(1, 32767)
            self.audio = audio.sink_make(32000)

            self.tb.connect(self.src, (self.mp4, 0), self.s2f_left, (self.audio, 0))
            self.tb.connect((self.mp4, 1), self.s2f_right, (self.audio, 1))
            self.tb.run()
        else:
            log = gr.logger("log")
            log.debug("debug file not found - skipped test")
            log.set_level("WARN")
        pass


if __name__ == '__main__':
    gr_unittest.run(qa_mp4_decode_bs, "qa_mp4_decode_bs.xml")



#!/usr/bin/env python
# _*_ coding: utf8 _*_

# Andreas Müller, 2008
# andrmuel@ee.ethz.ch
#
# this code may be freely used under GNU GPL conditions

"""
receive DAB with USRP
"""

from gnuradio import gr, blocks
import osmosdr
from gnuradio.eng_option import eng_option
import grdab
from optparse import OptionParser
import sys, time, threading, math

class rtl_sdr_dab_rx(gr.top_block):
	def __init__(self):
		gr.top_block.__init__(self)
        
		
		parser = OptionParser(option_class=eng_option, usage="%prog: [options] output-filename")
  		parser.add_option("-m", "--dab-mode", type="int", default=1,
        	     	help="DAB mode [default=%default]")
		parser.add_option("-F", "--filter-input", action="store_true", default=False,
                          help="Enable FFT filter at input")
		parser.add_option('-c', '--correct-ffe', action="store_true", default=False,
		     help="do fine frequency correction")
		parser.add_option('-u', '--correct-ffe-usrp', action="store_true", default=False,
		     help="do fine frequency correction by retuning the USRP instead of in software")
		parser.add_option('-e', '--equalize-magnitude', action="store_true", default=False,
		     help="do magnitude equalization")
  		parser.add_option("-s", "--resample-fixed", type="eng_float", default=1,
			help="resample by a fixed factor (fractional interpolation)")
		parser.add_option("-S", "--autocorrect-sample-rate", action="store_true", default=False,
                          help="Estimate sample rate offset and resample (dynamic fractional interpolation)")
		parser.add_option("-f", "--freq", type="eng_float", default=227.36e6,
		     help="set frequency to FREQ [default=%default]")
		parser.add_option("-p", "--ppm", type="int", default=0,
		     help="set frequency correction in ppm [default=%default]")
		parser.add_option("-r", "--sample-rate", type="int", default=2000000,
		     help="set sample rate to SAMPLE_RATE [default=%default]")
		parser.add_option("-g", "--rx-gain", type="eng_float", default=None,
		     help="set receive gain in dB (default is midpoint)")
		parser.add_option('-v', '--verbose', action="store_true", default=False,
		     help="verbose output")
        	(options, args) = parser.parse_args ()

		# if len(args)!=1:
			# parser.print_help()
			# sys.exit(1)
		# else:
			# self.filename = args[0]

		# if gr.enable_realtime_scheduling() != gr.RT_OK:
		#       print "-> failed to enable realtime scheduling"

		self.verbose = options.verbose

		self.sample_rate = sample_rate = options.sample_rate

		self.src = osmosdr.source()
		self.src.set_sample_rate(sample_rate)
		#self.src.set_center_freq(209.936e6, 0)
		self.src.set_freq_corr(options.ppm)
		self.src.set_gain_mode(True, 0)
		self.src.set_gain(0, 0)

		self.dab_params = grdab.parameters.dab_parameters(
                        mode=options.dab_mode,
                        sample_rate=self.sample_rate,
                        verbose=options.verbose
                        )
		self.rx_params = grdab.parameters.receiver_parameters(
                        mode=options.dab_mode,
                        softbits=True,
                        input_fft_filter=options.filter_input,
                        autocorrect_sample_rate=options.autocorrect_sample_rate,
                        sample_rate_correction_factor=options.resample_fixed,
                        verbose=options.verbose,
                        correct_ffe=options.correct_ffe,
                        equalize_magnitude=options.equalize_magnitude
                        )

		self.demod = grdab.ofdm_demod(self.dab_params, self.rx_params, verbose=options.verbose) 

                if len(args) >= 1:
                        self.filename = args[0]
                        self.sink = blocks.file_sink(gr.sizeof_char*12288, self.filename)
                        #self.sink = gr.file_sink(gr.sizeof_char*3072, self.filename)
                        self.connect(self.demod, self.sink)


		
		self.fic_dec = grdab.fic_decode(self.dab_params)
		self.connect(self.src, self.demod, self.fic_dec)

		# tune frequency
		self.frequency = options.freq
		self.set_freq(options.freq)

		# set gain      
		if options.rx_gain is None:
			# if no gain was specified, use AGC
                        self.src.set_gain_mode(True, 0)
                else:
                        self.src.set_gain(options.rx_gain, 0)

		self.update_ui = options.verbose
		if self.update_ui:
			self.run_ui_update_thread = True
			self.ui_updater = threading.Timer(0.1,self.update_ui_function)
			self.ui_updater.setDaemon(True)
			self.ui_updater.start()

		self.correct_ffe_usrp = options.correct_ffe_usrp
		if self.correct_ffe_usrp:
			print "--> correcting FFE on USRP"
			self.run_correct_ffe_thread = True
			self.ffe_updater = threading.Timer(0.1, self.correct_ffe)
			self.ffe_updater.setDaemon(True)
			self.ffe_updater.start()

	def update_ui_function(self):
		while self.run_ui_update_thread:
			var = self.demod.probe_phase_var.level()
			q = int(50*(math.sqrt(var)/(math.pi/4)))
			print "--> Phase variance: " + str(var) +"\n"
			print "--> Signal quality: " + '='*(50-q) + '>' + '-'*q + "\n"
			time.sleep(0.3)
	
	def correct_ffe(self):
		while self.run_correct_ffe_thread:
			diff = self.demod.sync.ffs_sample_and_average_arg.ffe_estimate()
			if abs(diff) > self.rx_params.usrp_ffc_min_deviation:
				self.frequency -= diff*self.rx_params.usrp_ffc_adapt_factor
				print "--> updating fine frequency correction: " + str(self.frequency)
				self.set_freq(self.frequency)
			time.sleep(1./self.rx_params.usrp_ffc_retune_frequency)

	def set_freq(self, freq):
		if self.src.set_center_freq(freq, 0):
			if self.verbose:
				print "--> retuned to " + str(freq) + " Hz"
			return True
		else:
			print "-> error - cannot tune to " + str(freq) + " Hz"
			return False
        
if __name__=='__main__':
	try:
		rx = rtl_sdr_dab_rx()
		rx.run()
	except KeyboardInterrupt:
		pass






#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab

class qa_fractional_interpolator_triggered_update(gr_unittest.TestCase):
	"""
	@brief Module test for the fractional interpolator block with triggered update.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_fractional_interpolator_triggered_update_cc(self):
		src_data = [0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1]
		trigger  = [0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
		expected_result = [3,4,5,6,7,8,0,2,4] # not sure why the values at the start and end are missing, but the trigger works nicely ..
		expected_result = [x + 0j for x in expected_result]
		src0 = blocks.vector_source_c(src_data)
		src1 = blocks.vector_source_b(trigger)
		fractional_interpolator_triggered_update = grdab.fractional_interpolator_triggered_update_cc(0,1)
		fractional_interpolator_triggered_update.set_interp_ratio(2)
		dst = blocks.vector_sink_c()
		self.tb.connect(src0, (fractional_interpolator_triggered_update,0))
		self.tb.connect(src1, (fractional_interpolator_triggered_update,1))
		self.tb.connect(fractional_interpolator_triggered_update, dst)
		self.tb.run()
		result_data = dst.data()
		# print src_data
		# print expected_result
		# print result_data
		self.assertComplexTuplesAlmostEqual(expected_result, result_data, 6)


if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest, blocks
import grdab

class qa_ofdm_insert_pilot_vcc(gr_unittest.TestCase):
	"""
	@brief Module test for the class that inserts the pilot symbol.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_ofdm_insert_pilot_vcc(self):
		pilot = [1j,2]
		src_data0        = (0,1,2,3,4,5,6,7,8,9,0,1,2,3)
		src_data1        = (1,0,0,1,0,1,0)
		expected_result0 = (1j,2,0,1,2,3,4,5,1j,2,6,7,8,9,1j,2,0,1,2,3)
		expected_result0 = [x+0j for x in expected_result0]
		expected_result1 = (1,0,0,0,1,0,0,1,0,0)
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		s2v0 = blocks.stream_to_vector(gr.sizeof_gr_complex,2)
		ofdm_insert_pilot = grdab.ofdm_insert_pilot_vcc(pilot)
		v2s0 = blocks.vector_to_stream(gr.sizeof_gr_complex,2)
		dst0 = blocks.vector_sink_c()
		dst1 = blocks.vector_sink_b()
		self.tb.connect(src0, s2v0, (ofdm_insert_pilot,0))
		self.tb.connect(src1, (ofdm_insert_pilot,1))
		self.tb.connect((ofdm_insert_pilot,0), v2s0, dst0)
		self.tb.connect((ofdm_insert_pilot,1), dst1)
		self.tb.run()
		result_data0 = dst0.data()
		result_data1 = dst1.data()
		self.assertComplexTuplesAlmostEqual(expected_result0, result_data0, 6)
		self.assertEqual(result_data1, expected_result1)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2018 Ruben Undheim
# (based on code from Moritz Luca Schmid, Communications Engineering Lab (CEL) / Karlsruhe Institute of Technology (KIT).)
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import gr, blocks
from gnuradio import filter
import grdab

class dab_audio_decoder_ff(gr.hier_block2):
    """
    """

    def __init__(self, dab_params, bit_rate, address, subch_size, protection, output_float, verbose=False, debug=False):
        if output_float: # map short samples to the range [-1,1] in floats
            gr.hier_block2.__init__(self,
                                    "dab_audio_decoder_ff",
                                    # Input signature
                                    gr.io_signature(1, 1, gr.sizeof_float * dab_params.num_carriers * 2),
                                    # Output signature
                                    gr.io_signature2(2, 2, gr.sizeof_float, gr.sizeof_float))
        else: # output signed 16 bit integers (directly from decoder)
            gr.hier_block2.__init__(self,
                                    "dab_audio_decoder_ff",
                                    # Input signature
                                    gr.io_signature(1, 1, gr.sizeof_float * dab_params.num_carriers * 2),
                                    # Output signature
                                    gr.io_signature2(2, 2, gr.sizeof_short, gr.sizeof_short))

        self.msc_dec = grdab.msc_decode(dab_params, address, subch_size, protection)
        self.unpack = blocks.packed_to_unpacked_bb_make(1, gr.GR_MSB_FIRST)
        self.mp2_dec = grdab.mp2_decode_bs_make(bit_rate / 8)
        self.connect((self, 0), self.msc_dec, self.unpack, self.mp2_dec)

        if output_float:
            # map short samples to the range [-1,1] in floats
            self.s2f_left = blocks.short_to_float_make(1, 32767)
            self.s2f_right = blocks.short_to_float_make(1, 32767)
            self.gain_left = blocks.multiply_const_ff(1, 1)
            self.gain_right = blocks.multiply_const_ff(1, 1)
            self.connect((self.mp2_dec, 0), self.s2f_left, self.gain_left, (self, 0))
            self.connect((self.mp2_dec, 1), self.s2f_right, self.gain_right, (self, 1))
        else:
            # output signed 16 bit integers (directly from decoder)
            self.connect((self.mp2_dec, 0), (self, 0))
            self.connect((self.mp2_dec, 1), (self, 1))

    def set_volume(self, volume):
        self.gain_left.set_k(volume)
        self.gain_right.set_k(volume)

    def get_sample_rate(self):
        return self.mp2_dec.get_sample_rate()



#!/usr/bin/env python
#
# Copyright 2008 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

# test_ofdm.py - test the ofdm demod block
#
# Andreas Mueller, 2008
# andrmuel@ee.ethz.ch

from gnuradio import gr, blocks
from gnuradio.eng_option import eng_option
from optparse import OptionParser
from dab import ofdm
import parameters
import random

class test_ofdm(gr.top_block):
	"""
	@brief Test program for the ofdm_demod class.
	"""
	def __init__(self):
		gr.top_block.__init__(self)
		

		usage = "%prog: [options] samples_file"
		parser = OptionParser(option_class=eng_option, usage=usage)
  		parser.add_option("-m", "--dab-mode", type="int", default=1,
        	     	help="DAB mode [default=%default]")
		parser.add_option("-F", "--filter-input", action="store_true", default=False,
                          help="Enable FFT filter at input")
  		parser.add_option("-s", "--resample-fixed", type="float", default=1,
			help="resample by a fixed factor (fractional interpolation)")
		parser.add_option("-S", "--autocorrect-sample-rate", action="store_true", default=False,
                          help="Estimate sample rate offset and resample (dynamic fractional interpolation)")
  		parser.add_option('-r', '--sample-rate', type="int", default=2048000,
	     		help="Use non-standard sample rate (default=%default)")
  		parser.add_option('-e', '--equalize-magnitude', action="store_true", default=False,
	     		help="Enable individual carrier magnitude equalizer")
  		parser.add_option('-d', '--debug', action="store_true", default=False,
	     		help="Write output to files")
  		parser.add_option('-v', '--verbose', action="store_true", default=False,
	     		help="Print status messages")
		(options, args) = parser.parse_args ()
	
		dp = parameters.dab_parameters(options.dab_mode, verbose=options.verbose, sample_rate=options.sample_rate)

		rp = parameters.receiver_parameters(options.dab_mode, input_fft_filter=options.filter_input, autocorrect_sample_rate=options.autocorrect_sample_rate, sample_rate_correction_factor=options.resample_fixed, equalize_magnitude=options.equalize_magnitude, verbose=options.verbose)

		if len(args)<1:
			if options.verbose: print "-> using repeating random vector as source"
			self.sigsrc = blocks.vector_source_c([10e6*(random.random() + 1j*random.random()) for i in range(0,100000)],True)
			self.ns_simulate = blocks.vector_source_c([0.01]*dp.ns_length+[1]*dp.symbols_per_frame*dp.symbol_length,1)
			self.mult = blocks.multiply_cc() # simulate null symbols ...
			self.src = blocks.throttle( gr.sizeof_gr_complex,2048000)
			self.connect(self.sigsrc, (self.mult, 0))
			self.connect(self.ns_simulate, (self.mult, 1))
			self.connect(self.mult, self.src)
		else:
			filename = args[0]
			if options.verbose: print "-> using samples from file " + filename
			self.src = blocks.file_source(gr.sizeof_gr_complex, filename, False)

		self.dab_demod = ofdm.ofdm_demod(dp, rp, debug=options.debug, verbose=options.verbose)
		
		self.connect(self.src, self.dab_demod)

		# sink output to nowhere 
		self.nop0 = blocks.nop(gr.sizeof_char*dp.num_carriers/4)
		self.nop1 = blocks.nop(gr.sizeof_char)
		self.connect((self.dab_demod,0),self.nop0)
		self.connect((self.dab_demod,1),self.nop1)
			
if __name__=='__main__':
	try:
		to = test_ofdm()
		# to.run()
		to.run()
		to.dump()
		to.dab_demod.stop()
		
	except KeyboardInterrupt:
		pass




# Copyright 2008 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

# Andreas Mueller, 2008
# andrmuel@ee.ethz.ch
# Moritz Luca Schmid, 2017
# luca.m.schmid@gmail.com

from gnuradio import gr, blocks
from grdab import grdab_swig

class detect_null(gr.hier_block2):
	"""
	@brief Detect Null symbols by looking at the energy of the symbol

	This block outputs a 1 where a new DAB frame begins, and zeros otherwise.
	"""
	
	def __init__(self, length, debug=False):
		"""
		Hierarchical block to detect Null symbols

		@param length length of the Null symbol (in samples)
		@param debug whether to write signals out to files
		"""
		gr.hier_block2.__init__(self,"detect_null",
		                        gr.io_signature(1, 1, gr.sizeof_gr_complex),    # input signature
					gr.io_signature(1, 1, gr.sizeof_char))          # output signature


		# get the magnitude squared
		self.ns_c2magsquared = blocks.complex_to_mag_squared()
		
		# this wastes cpu cycles:
		# ns_detect_taps = [1]*length
		# self.ns_moving_sum = gr.fir_filter_fff(1,ns_detect_taps)
		# this isn't better:
		#self.ns_filter = gr.iir_filter_ffd([1]+[0]*(length-1)+[-1],[0,1])
		# this does the same again, but is actually faster (outsourced to an independent block ..):
		self.ns_moving_sum = grdab_swig.moving_sum_ff(length)
		self.ns_invert = blocks.multiply_const_ff(-1)

		# peak detector on the inverted, summed up signal -> we get the nulls (i.e. the position of the start of a frame)
		self.ns_peak_detect = grdab_swig.peak_detector_fb(0.6,0.7,10,0.0001) # mostly found by try and error -> remember that the values are negative!

		# connect it all
		self.connect(self, self.ns_c2magsquared, self.ns_moving_sum, self.ns_invert, self.ns_peak_detect, self)

		if debug:
			self.connect(self.ns_invert, blocks.file_sink(gr.sizeof_float, "debug/ofdm_sync_dab_ns_filter_inv_f.dat"))
			self.connect(self.ns_peak_detect,blocks.file_sink(gr.sizeof_char, "debug/ofdm_sync_dab_peak_detect_b.dat"))
	
	def clear_state(self):	
		self.ns_moving_sum.reset()
		self.ns_peak_detect.clear_state()



#!/usr/bin/env python
#
# Copyright 2008 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

# test_ofdm.py - test the ofdm demod block
#
# Andreas Mueller, 2008
# andrmuel@ee.ethz.ch

from gnuradio import gr
from gnuradio.eng_option import eng_option
import grdab
from optparse import OptionParser
import sys

class test_fic(gr.top_block):
	"""
	@brief Test program for the fic_decode class.
	"""
	def __init__(self):
		gr.top_block.__init__(self)

		usage = "%prog: [options] samples_file"
		parser = OptionParser(option_class=eng_option, usage=usage)
  		parser.add_option("-m", "--dab-mode", type="int", default=1,
        	     	help="DAB mode [default=%default]")
		parser.add_option("-F", "--filter-input", action="store_true", default=False,
                          help="Enable FFT filter at input")
  		parser.add_option("-s", "--resample-fixed", type="eng_float", default=1,
			help="resample by a fixed factor (fractional interpolation)")
		parser.add_option("-S", "--autocorrect-sample-rate", action="store_true", default=False,
                          help="Estimate sample rate offset and resample (dynamic fractional interpolation)")
  		parser.add_option('-u', '--usrp-source', action="store_true", default=False,
	     		help="Samples from USRP (-> adjust params for 2 MSPS)")
  		parser.add_option('-d', '--debug', action="store_true", default=False,
	     		help="Write output to files")
  		parser.add_option('-v', '--verbose', action="store_true", default=False,
	     		help="Print status messages")
		(options, args) = parser.parse_args ()
	
		if options.usrp_source:
			dp = grdab.dab_parameters(options.dab_mode, verbose=options.verbose, sample_rate=2000000)
		else:
			dp = grdab.dab_parameters(options.dab_mode, verbose=options.verbose)

		rp = grdab.receiver_parameters(options.dab_mode, softbits=True, input_fft_filter=options.filter_input, autocorrect_sample_rate=options.autocorrect_sample_rate, sample_rate_correction_factor=options.resample_fixed, correct_ffe=True, equalize_magnitude=True, verbose=options.verbose)

		if len(args)<1:
			print "error: need file with samples"
			sys.exit(1)
		else:
			filename = args[0]
			if options.verbose: print "--> using samples from file " + filename
			self.src = gr.file_source(gr.sizeof_gr_complex, filename, False)

		
		self.dab_demod = grdab.ofdm_demod(dp, rp, verbose=options.verbose, debug=options.debug)
		self.fic_dec   = grdab.fic_decode(dp, verbose=options.verbose, debug=options.debug)
		
		self.connect(self.src, self.dab_demod, self.fic_dec)
		self.connect((self.dab_demod,1), (self.fic_dec,1))

if __name__=='__main__':
	try:
		tf = test_fic()
		tf.run()
		tf.dab_demod.stop()
		print "processing rate: " + str(tf.dab_demod.measure_rate.processing_rate())
		print "bitrate: " + str(tf.dab_demod.measure_rate.bitrate())
	except KeyboardInterrupt:
		pass




#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab

class qa_measure_ber_b(gr_unittest.TestCase):
	"""
	@brief QA for bit error rate measure sink.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_measure_ber_b(self):
		src0 = blocks.vector_source_b(range(0,256))
		src1 = blocks.vector_source_b(range(0,256))
		sink = grdab.measure_ber_b()
		self.tb.connect(src0, (sink,0))
		self.tb.connect(src1, (sink,1))
		self.tb.run()
		assert(sink.ber()==0)
	
	def test_001_measure_ber_b(self):
		src0 = blocks.vector_source_b([0]*100)
		src1 = blocks.vector_source_b([255]*100)
		sink = grdab.measure_ber_b()
		self.tb.connect(src0, (sink,0))
		self.tb.connect(src1, (sink,1))
		self.tb.run()
		assert(sink.ber()==1)

	def test_002_measure_ber_b(self):
		src0 = blocks.vector_source_b(range(0,256)*2)
		src1 = blocks.vector_source_b([0]*256+[255]*256)
		sink = grdab.measure_ber_b()
		self.tb.connect(src0, (sink,0))
		self.tb.connect(src1, (sink,1))
		self.tb.run()
		self.assertAlmostEqual(sink.ber(), 0.5) # every bit is set 50% of the time

	def test_003_measure_ber_b(self):
		src0 = blocks.vector_source_b([0,7,255,250])
		src1 = blocks.vector_source_b([0,2,255,3])
		sink = grdab.measure_ber_b()
		self.tb.connect(src0, (sink,0))
		self.tb.connect(src1, (sink,1))
		self.tb.run()
		self.assertAlmostEqual(sink.ber(), 8./32.) # every bit is set 50% of the time

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab

class qa_complex_to_interleaved_float_vcf(gr_unittest.TestCase):
	"""
	@brief QA for the complex to interleaved float block

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_complex_to_interleaved_float_vcf(self):
		src_data        = (1+2j,3+4j,5+6j,7+8j)
		expected_result = (1,3,2,4,5,7,6,8)
		src = blocks.vector_source_c(src_data)
		s2v = blocks.stream_to_vector(gr.sizeof_gr_complex, 2)
		complex_to_interleaved_float_vcf = grdab.complex_to_interleaved_float_vcf(2)
		v2s = blocks.vector_to_stream(gr.sizeof_float, 4)
		dst = blocks.vector_sink_f()
		self.tb.connect(src, s2v, complex_to_interleaved_float_vcf, v2s, dst)
		self.tb.run()
		result_data = dst.data()
		# print result_data
		self.assertFloatTuplesAlmostEqual(expected_result, result_data, 6)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2017 Moritz Luca Schmid, Communications Engineering Lab (CEL) / Karlsruhe Institute of Technology (KIT).
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import grdab_swig as grdab
import os

class qa_reed_solomon_decode_bb (gr_unittest.TestCase):

    def setUp (self):
        self.tb = gr.top_block ()

    def tearDown (self):
        self.tb = None

# insert 5 errors in rs-encoded prbs reference sequence and correct them with rs_decoder
    def test_001_t(self):
        self.prbs = (
        154, 15, 22, 223, 146, 92, 238, 15, 39, 87, 230, 120, 80, 186, 147, 176, 169, 49, 253, 117, 245, 122, 30, 187,
        74, 141, 148, 1, 181, 10, 0, 244, 250, 199, 227, 56, 155, 105, 187, 219, 135, 61, 241, 87, 223, 75, 59, 112, 78,
        238, 63, 69, 246, 177, 92, 140, 117, 34, 254, 70, 18, 131, 116, 13, 51, 174, 239, 86, 135, 157, 180, 97, 156,
        48, 179, 190, 218, 99, 171, 29, 49, 42, 78, 63, 3, 7, 3, 145, 60, 180, 134, 27, 104, 230, 32, 171, 6, 109, 106,
        1, 6, 45, 104, 206, 138, 38, 107, 242, 128, 228)

        self.corrupted_data = (
        1, 1, 1, 1, 1, 92, 238, 15, 39, 87, 230, 120, 80, 186, 147, 176, 169, 49, 253, 117, 245, 122,
        30, 187, 74, 141, 148, 1, 181, 10, 0, 244, 250, 199, 227, 56, 155, 105, 187, 219, 135, 61,
        241, 87, 223, 75, 59, 112, 78, 238, 63, 69, 246, 177, 92, 140, 117, 34, 254, 70, 18, 131,
        116, 13, 51, 174, 239, 86, 135, 157, 180, 97, 156, 48, 179, 190, 218, 99, 171, 29, 49, 42,
        78, 63, 3, 7, 3, 145, 60, 180, 134, 27, 104, 230, 32, 171, 6, 109, 106, 1, 6, 45, 104,
        206, 138, 38, 107, 242, 128, 228, 215, 34, 43, 109, 122, 92, 195, 54, 105, 246)

        self.src = blocks.vector_source_b(self.corrupted_data)
        self.rs_decoder = grdab.reed_solomon_decode_bb_make(1)
        self.sink = blocks.vector_sink_b_make()
        self.tb.connect(self.src, self.rs_decoder, self.sink)
        self.tb.run()
        data = self.sink.data()
        self.assertEqual(data, self.prbs)


if __name__ == '__main__':
    gr_unittest.run(qa_reed_solomon_decode_bb, "qa_reed_solomon_decode_bb.xml")



#!/usr/bin/env python2
# -*- coding: utf8 -*-

# Andreas Müller, 2008
# andrmuel@ee.ethz.ch
#
# this code may be freely used under GNU GPL conditions

"""
receive DAB with USRP
"""

from gnuradio import gr, uhd, blocks
from gnuradio import audio
import grdab
from gnuradio.eng_option import eng_option
from optparse import OptionParser
import sys, time, threading, math

class usrp_dab_rx(gr.top_block):
	def __init__(self):
		gr.top_block.__init__(self)
        
		
		parser = OptionParser(option_class=eng_option, usage="%prog: [options] output-filename")
  		parser.add_option("-m", "--dab-mode", type="int", default=1,
        	     	help="DAB mode [default=%default]")
		parser.add_option("-F", "--filter-input", action="store_true", default=False,
                          help="Enable FFT filter at input")
		parser.add_option('-c', '--correct-ffe', action="store_true", default=False,
		     help="do fine frequency correction")
		parser.add_option('-u', '--correct-ffe-usrp', action="store_true", default=False,
		     help="do fine frequency correction by retuning the USRP instead of in software")
		parser.add_option('-e', '--equalize-magnitude', action="store_true", default=False,
		     help="do magnitude equalization")
  		parser.add_option("-s", "--resample-fixed", type="eng_float", default=1,
			help="resample by a fixed factor (fractional interpolation)")
		parser.add_option("-S", "--autocorrect-sample-rate", action="store_true", default=False,
                          help="Estimate sample rate offset and resample (dynamic fractional interpolation)")
		parser.add_option("-R", "--rx-subdev-spec", type="subdev", default=(0, 0),
		     help="select USRP Rx side A or B [default=A]")
		parser.add_option("-f", "--freq", type="eng_float", default=227.36e6,
		     help="set frequency to FREQ [default=%default]")
		parser.add_option("-d", "--decim", type="intx", default=32,
		     help="set decimation rate to DECIM [default=%default]")
		parser.add_option("-g", "--rx-gain", type="eng_float", default=None,
		     help="set receive gain in dB (default is midpoint)")
		parser.add_option('-v', '--verbose', action="store_true", default=False,
		     help="verbose output")
		parser.add_option('-a', '--antenna', type="string", default="TX/RX",
		     help="select antenna")
        	(options, args) = parser.parse_args ()

		# if len(args)!=1:
			# parser.print_help()
			# sys.exit(1)
		# else:
			# self.filename = args[0]

		# if gr.enable_realtime_scheduling() != gr.RT_OK:
		#       print "-> failed to enable realtime scheduling"

		self.verbose = options.verbose

		self.src = uhd.usrp_source("",uhd.io_type.COMPLEX_FLOAT32,1)
        	#self.src.set_mux(usrp.determine_rx_mux_value(self.src, options.rx_subdev_spec))
        	#self.subdev = uhd.selected_subdev(self.src, options.rx_subdev_spec)
        	#print "--> using RX dboard " + self.subdev.side_and_name()
		
		self.sample_rate = 2e6#self.src.adc_rate()/options.decim
		self.src.set_samp_rate(self.sample_rate)
		self.src.set_antenna(options.antenna)
		self.dab_params = grdab.parameters.dab_parameters(mode=options.dab_mode, sample_rate=self.sample_rate, verbose=options.verbose)
		self.rx_params = grdab.parameters.receiver_parameters(mode=options.dab_mode, softbits=True, input_fft_filter=options.filter_input, autocorrect_sample_rate=options.autocorrect_sample_rate, sample_rate_correction_factor=options.resample_fixed, verbose=options.verbose, correct_ffe=options.correct_ffe, equalize_magnitude=options.equalize_magnitude)

		self.demod = grdab.ofdm_demod(self.dab_params, self.rx_params, verbose=options.verbose) 

		# self.sink = gr.file_sink(gr.sizeof_char*384, self.filename)
		# self.trigsink = gr.null_sink(gr.sizeof_char)
		# self.connect(self.src, self.demod, self.sink)
		# self.connect((self.demod,1), self.trigsink)
		
		self.fic_dec = grdab.fic_decode(self.dab_params)
		self.connect(self.src, self.demod, self.fic_dec)

		# add MSC chain
		self.dabplus = grdab.dabplus_audio_decoder_ff(self.dab_params, 112, 54, 84, 2, True)
		self.audio = audio.sink_make(32000)
		self.connect(self.demod, self.dabplus)
		# left stereo channel
		self.connect((self.dabplus, 0), (self.audio, 0))
		# right stereo channel
		self.connect((self.dabplus, 1), (self.audio, 1))		


		# tune frequency
		self.frequency = options.freq
		self.set_freq(options.freq)

		# set gain      
		if options.rx_gain is None:
			# if no gain was specified, use the mid-point in dB
			g = self.src.get_gain_range()
			options.rx_gain = float(g.start()+g.stop())/2
		self.src.set_gain(options.rx_gain)
		#self.subdev.set_gain(options.rx_gain)

		self.update_ui = options.verbose
		if self.update_ui:
			self.run_ui_update_thread = True
			self.ui_updater = threading.Timer(0.1,self.update_ui_function)
			self.ui_updater.setDaemon(True)
			self.ui_updater.start()

		self.correct_ffe_usrp = options.correct_ffe_usrp
		if self.correct_ffe_usrp:
			print "--> correcting FFE on USRP"
			self.run_correct_ffe_thread = True
			self.ffe_updater = threading.Timer(0.1, self.correct_ffe)
			self.ffe_updater.setDaemon(True)
			self.ffe_updater.start()

	def update_ui_function(self):
		while self.run_ui_update_thread:
			var = self.demod.probe_phase_var.level()
			q = int(50*(math.sqrt(var)/(math.pi/4)))
			print "--> Phase variance: " + str(var) +"\n"
			print "--> Signal quality: " + '='*(50-q) + '>' + '-'*q + "\n"
			time.sleep(0.3)
	
	def correct_ffe(self):
		while self.run_correct_ffe_thread:
			diff = self.demod.sync.ffs_sample_and_average_arg.ffe_estimate()
			if abs(diff) > self.rx_params.usrp_ffc_min_deviation:
				self.frequency -= diff*self.rx_params.usrp_ffc_adapt_factor
				print "--> updating fine frequency correction: " + str(self.frequency)
				self.set_freq(self.frequency)
			time.sleep(1./self.rx_params.usrp_ffc_retune_frequency)

	def set_freq(self, freq):
		if self.src.set_center_freq(freq): #src.tune(0, self.subdev, freq):
			if self.verbose:
				print "--> retuned to " + str(freq) + " Hz"
			return True
		else:
			print "-> error - cannot tune to " + str(freq) + " Hz"
			return False
        
if __name__=='__main__':
	try:
		rx = usrp_dab_rx()
		rx.run()
	except KeyboardInterrupt:
		pass






#!/usr/bin/env python

from gnuradio import gr, gr_unittest, blocks
import grdab

class qa_ofdm_sampler(gr_unittest.TestCase):
	"""
	@brief Module test for the OFDM sampler.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_ofdm_sampler(self):
		fft_len = 3
		src_data0 = (0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0)
		src_data1 = (0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0)
		expected_result0 = (4,5,6,9,0,1,6,7,8)
		expected_result0 = [x+0j for x in expected_result0]
		expected_result1 = (1,0,1)
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		ofdm_sampler = grdab.ofdm_sampler(fft_len,2,2,0)
		v2s0 = blocks.vector_to_stream(gr.sizeof_gr_complex,fft_len)
		dst0 = blocks.vector_sink_c()
		dst1 = blocks.vector_sink_b()
		self.tb.connect(src0, (ofdm_sampler,0))
		self.tb.connect(src1, (ofdm_sampler,1))
		self.tb.connect((ofdm_sampler,0), v2s0, dst0)
		self.tb.connect((ofdm_sampler,1), dst1)
		self.tb.run()
		result_data0 = dst0.data()
		result_data1 = dst1.data()
		self.assertComplexTuplesAlmostEqual(expected_result0, result_data0, 6)
		self.assertEqual(result_data1, expected_result1)

	def test_002_ofdm_sampler(self):
		fft_len = 3
		src_data0 = (0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0)
		src_data1 = (0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0)
		expected_result0 = (3,4,5,8,9,0,5,6,7)
		expected_result0 = [x+0j for x in expected_result0]
		expected_result1 = (1,0,1)
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		ofdm_sampler = grdab.ofdm_sampler(fft_len,2,2,1)
		v2s0 = blocks.vector_to_stream(gr.sizeof_gr_complex,fft_len)
		dst0 = blocks.vector_sink_c()
		dst1 = blocks.vector_sink_b()
		self.tb.connect(src0, (ofdm_sampler,0))
		self.tb.connect(src1, (ofdm_sampler,1))
		self.tb.connect((ofdm_sampler,0), v2s0, dst0)
		self.tb.connect((ofdm_sampler,1), dst1)
		self.tb.run()
		result_data0 = dst0.data()
		result_data1 = dst1.data()
		self.assertComplexTuplesAlmostEqual(expected_result0, result_data0, 6)
		self.assertEqual(result_data1, expected_result1)

	def test_003_ofdm_sampler(self):
		fft_len = 4
		src_data0 = (0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0)
		src_data1 = (1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)
		expected_result0 = (0,1,2,3,7,8,9,0,4,5,6,7)
		expected_result0 = [x+0j for x in expected_result0]
		expected_result1 = (1,0,0)
		src0 = blocks.vector_source_c(src_data0)
		src1 = blocks.vector_source_b(src_data1)
		ofdm_sampler = grdab.ofdm_sampler(fft_len,3,5,3)
		v2s0 = blocks.vector_to_stream(gr.sizeof_gr_complex,fft_len)
		dst0 = blocks.vector_sink_c()
		dst1 = blocks.vector_sink_b()
		self.tb.connect(src0, (ofdm_sampler,0))
		self.tb.connect(src1, (ofdm_sampler,1))
		self.tb.connect((ofdm_sampler,0), v2s0, dst0)
		self.tb.connect((ofdm_sampler,1), dst1)
		self.tb.run()
		result_data0 = dst0.data()
		result_data1 = dst1.data()
		self.assertComplexTuplesAlmostEqual(expected_result0, result_data0, 6)
		self.assertEqual(result_data1, expected_result1)

if __name__ == '__main__':
	gr_unittest.main()




#!/usr/bin/env python

from gnuradio import gr, gr_unittest
import dab_swig

class qa_ofdm_coarse_frequency_correct(gr_unittest.TestCase):
	"""
	@brief QA for the coarse frequency correction class.

	This class implements a test bench to verify the corresponding C++ class.
	"""

	def setUp(self):
		self.tb = gr.top_block()

	def tearDown(self):
		self.tb = None

	def test_001_ofdm_coarse_frequency_correct(self):
		src_data0        = [0,1,2,3,4,5,6,7,8,9,1,2,0,5,7,6,0,4,0,6,1,1,1,0.8,0.1,1.3,1,0.7,1,1,0,1,2,3,4,5,6,7,8,9]
		expected_result0 = [7,9,5,6,0.8,1.3,3,5]
		expected_result0 = [complex(x) for x in expected_result0]
		src_data1 = [1,1,1,0]
		expected_result1 = (1,1,1,0)
		src0 = gr.vector_source_c(src_data0)
		src1 = gr.vector_source_b(src_data1)
		s2v0 = gr.stream_to_vector(gr.sizeof_gr_complex, 10)
		ofdm_coarse_frequency_correct = dab_swig.ofdm_coarse_frequency_correct(10,2)
		v2s0 = gr.vector_to_stream(gr.sizeof_gr_complex, 2)
		dst0 = gr.vector_sink_c()
		dst1 = gr.vector_sink_b()
		self.tb.connect(src0, s2v0, (ofdm_coarse_frequency_correct,0))
		self.tb.connect(src1, (ofdm_coarse_frequency_correct,1))
		self.tb.connect((ofdm_coarse_frequency_correct,0), v2s0, dst0)
		self.tb.connect((ofdm_coarse_frequency_correct,1), dst1)
		self.tb.run()
		result_data0 = dst0.data()
		result_data1 = dst1.data()
		# print expected_result0
		# print result_data0
		self.assertComplexTuplesAlmostEqual(expected_result0, result_data0, 6)
		self.assertEqual(result_data1, expected_result1)

if __name__ == '__main__':
	gr_unittest.main()

